@@ -41,6 +41,11 @@
 #include <stdlib.h>
 #endif
 
+#ifdef __WIN32__
+# include <windows.h>
+# define sleep Sleep
+# endif
+
 #ifndef  __STDIO_H
 #include <stdio.h>
 #endif
@@ -402,7 +407,11 @@ pstr p;
     if (gzipIO==ARCH_GZIP)  {
 #ifndef _MVS
       p = NULL;
+#ifdef __WIN32__
+      CreateConcat  ( p,gzip_path,pstr(" | cat > "),FName );
+#else
       CreateConcat  ( p,gzip_path,pstr(" > "),FName );
+#endif
       hFile = popen ( p,"w" );
       if (p)  delete[] p;
 #else
