@@ -1,49 +1,24 @@
-//  $Id: linalg_.cpp,v 1.5 2005/01/20 17:27:07 keb Exp $
+//  $Id: linalg_.cpp,v 1.8 2008/07/08 15:28:14 keb Exp $
 //  =================================================================
 //
 //   CCP4 Coordinate Library: support of coordinate-related
 //   functionality in protein crystallography applications.
 //
-//   Copyright (C) Eugene Krissinel 2004.
+//   Copyright (C) Eugene Krissinel 2000-2008.
 //
-//   This library is free software and is distributed under the terms
-//   and conditions of the CCP4 licence agreement as `Part 0' (Annex 2)
-//   software, which is version 2.1 of the GNU Lesser General Public
-//   Licence (LGPL) with the following additional clause:
+//    This library is free software: you can redistribute it and/or 
+//    modify it under the terms of the GNU Lesser General Public 
+//    License version 3, modified in accordance with the provisions 
+//    of the license to address the requirements of UK law.
 //
-//      `You may also combine or link a "work that uses the Library"
-//      to produce a work containing portions of the Library, and
-//      distribute that work under terms of your choice, provided that
-//      you give prominent notice with each copy of the work that the
-//      specified version of the Library is used in it, and that you
-//      include or provide public access to the complete corresponding
-//      machine-readable source code for the Library including whatever
-//      changes were used in the work. (i.e. If you make changes to the
-//      Library you must distribute those, but you do not need to
-//      distribute source or object code to those portions of the work
-//      not covered by this licence.)'
+//    You should have received a copy of the modified GNU Lesser 
+//    General Public License along with this library. If not, copies 
+//    may be downloaded from http://www.ccp4.ac.uk/ccp4license.php
 //
-//   Note that this clause grants an additional right and does not
-//   impose any additional restriction, and so does not affect
-//   compatibility with the GNU General Public Licence (GPL). If you
-//   wish to negotiate other terms, please contact the maintainer.
-//
-//   You can redistribute it and/or modify the library under the terms
-//   of the GNU Lesser General Public License as published by the Free
-//   Software Foundation; either version 2.1 of the License, or (at
-//   your option) any later version.
-//
-//   This library is distributed in the hope that it will be useful,
-//   but WITHOUT ANY WARRANTY; without even the implied warranty of
-//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-//   Lesser General Public License for more details.
-//
-//   You should have received a copy of the CCP4 licence and/or GNU
-//   Lesser General Public License along with this library; if not,
-//   write to the CCP4 Secretary, Daresbury Laboratory, Warrington
-//   WA4 4AD, UK. The GNU Lesser General Public can also be obtained
-//   by writing to the Free Software Foundation, Inc., 59 Temple Place,
-//   Suite 330, Boston, MA 02111-1307 USA
+//    This program is distributed in the hope that it will be useful,
+//    but WITHOUT ANY WARRANTY; without even the implied warranty of
+//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+//    GNU Lesser General Public License for more details.
 //
 //  =================================================================
 //
@@ -56,7 +31,7 @@
 //  **** Project :  MMDB  ( MacroMolecular Data Base )
 //       ~~~~~~~~~
 //
-//  E. Krissinel 2000-2002
+//  (C) E. Krissinel 2000-2008
 //
 //  =================================================================
 //
@@ -553,7 +528,7 @@ void  SVD ( int    NA,    int     M,    int N,
 //     MatU -  compute  U , if set True
 //     MatV -  compute  V , if set True
 //     RV1  -  temporary array [1..N].
-//     U    - should be always suuplied as an array of
+//     U    - should be always supplied as an array of
 //            [1..M][1..N], M>=N .
 //     V    - should be suuplied as an array of
 //            [1..N][1..N] if MatV is True .
@@ -716,6 +691,7 @@ realtype  C,G,F,X,S,H,Y,Z,Scale,ANorm,GG;
 
     }
 
+
   //   Accumulation of the left-hand transformations
 
   if (MatU)  {
@@ -767,7 +743,13 @@ realtype  C,G,F,X,S,H,Y,Z,Scale,ANorm,GG;
         }
       }
 
-      if (ExitKey!=1)  {
+//      if (ExitKey!=1)  {  <-- this is original statement
+      if (ExitKey>1)  {  // <-- prevents from corruption due to l1<1.
+                         // This is a rare case as RV1[1] should be
+                         // always 0.0 . Apparently this logics is
+                         // on the edge of float-point arithmetic,
+                         // therefore extra precaution for the case
+                         // of l1<1 was found necessary.
         C       = 0.0;
         S       = 1.0;
         ExitKey = 0;
