@@ -1,60 +1,35 @@
-//  $Id: mattype_.cpp,v 1.10 2005/09/22 16:32:38 keb Exp $
+//  $Id: mattype_.cpp,v 1.12 2008/07/08 15:28:14 keb Exp $
 //  =================================================================
 //
 //   CCP4 Coordinate Library: support of coordinate-related
 //   functionality in protein crystallography applications.
 //
-//   Copyright (C) Eugene Krissinel 2004.
+//   Copyright (C) Eugene Krissinel 2000-2008.
 //
-//   This library is free software and is distributed under the terms
-//   and conditions of the CCP4 licence agreement as `Part 0' (Annex 2)
-//   software, which is version 2.1 of the GNU Lesser General Public
-//   Licence (LGPL) with the following additional clause:
+//    This library is free software: you can redistribute it and/or 
+//    modify it under the terms of the GNU Lesser General Public 
+//    License version 3, modified in accordance with the provisions 
+//    of the license to address the requirements of UK law.
 //
-//      `You may also combine or link a "work that uses the Library"
-//      to produce a work containing portions of the Library, and
-//      distribute that work under terms of your choice, provided that
-//      you give prominent notice with each copy of the work that the
-//      specified version of the Library is used in it, and that you
-//      include or provide public access to the complete corresponding
-//      machine-readable source code for the Library including whatever
-//      changes were used in the work. (i.e. If you make changes to the
-//      Library you must distribute those, but you do not need to
-//      distribute source or object code to those portions of the work
-//      not covered by this licence.)'
+//    You should have received a copy of the modified GNU Lesser 
+//    General Public License along with this library. If not, copies 
+//    may be downloaded from http://www.ccp4.ac.uk/ccp4license.php
 //
-//   Note that this clause grants an additional right and does not
-//   impose any additional restriction, and so does not affect
-//   compatibility with the GNU General Public Licence (GPL). If you
-//   wish to negotiate other terms, please contact the maintainer.
-//
-//   You can redistribute it and/or modify the library under the terms
-//   of the GNU Lesser General Public License as published by the Free
-//   Software Foundation; either version 2.1 of the License, or (at
-//   your option) any later version.
-//
-//   This library is distributed in the hope that it will be useful,
-//   but WITHOUT ANY WARRANTY; without even the implied warranty of
-//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-//   Lesser General Public License for more details.
-//
-//   You should have received a copy of the CCP4 licence and/or GNU
-//   Lesser General Public License along with this library; if not,
-//   write to the CCP4 Secretary, Daresbury Laboratory, Warrington
-//   WA4 4AD, UK. The GNU Lesser General Public can also be obtained
-//   by writing to the Free Software Foundation, Inc., 59 Temple Place,
-//   Suite 330, Boston, MA 02111-1307 USA
+//    This program is distributed in the hope that it will be useful,
+//    but WITHOUT ANY WARRANTY; without even the implied warranty of
+//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+//    GNU Lesser General Public License for more details.
 //
 //  =================================================================
 //
-//    17.06.03   <--  Date of Last Modification.
+//    28.07.06   <--  Date of Last Modification.
 //                   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //  -----------------------------------------------------------------
 //
 //  **** Module  :  MatType_ <implementation>
 //       ~~~~~~~~~
 //
-//  E. Krissinel 2000-2003
+//  (C) E. Krissinel 2000-2008
 //
 //  =================================================================
 //
@@ -693,7 +668,7 @@ Boolean Odd ( int i )  {
 
 //  ----------------------------------------------------
 
-long  HexValL ( const pstr S )  {
+long  HexValL ( cpstr S )  {
 char  C;
 int   i;
 long  z=0;
@@ -709,7 +684,7 @@ long  z=0;
 
 //  ----------------------------------------------------
 
-long  OctValL ( const pstr S )  {
+long  OctValL ( cpstr S )  {
 int   i;
 long  z=0;
   for (i=0;S[i];i++)  {
@@ -722,7 +697,7 @@ long  z=0;
 
 //  ----------------------------------------------------
 
-long  BinValL ( const pstr S )  {
+long  BinValL ( cpstr S )  {
 int   i;
 long  z=0;
   for (i=0;S[i];i++)  {
@@ -749,7 +724,7 @@ long  z;
 
 //  ----------------------------------------------------
 
-pstr ParamStr ( pstr D, const pstr S, realtype V, int M, const pstr S1 )  {
+pstr ParamStr ( pstr D, cpstr S, realtype V, int M, cpstr S1 )  {
 char  VS[30];
   strcat  ( D,S );
   sprintf ( VS,"%-.*g",M,V );
@@ -758,8 +733,8 @@ char  VS[30];
 }
 
 
-pstr ParamStr ( pstr D,  const pstr S, realtype V, int M,
-                const pstr S1, realtype V2, int M2, const pstr S2 )  {
+pstr ParamStr ( pstr D,  cpstr S, realtype V, int M,
+                cpstr S1, realtype V2, int M2, cpstr S2 )  {
 char  VS[30];
   ParamStr ( D,S,V,M,S1 );
   sprintf  ( VS,"%-.*g",M2,V2 );
@@ -768,7 +743,7 @@ char  VS[30];
 }
 
 
-pstr CreateCopy ( pstr & Dest, const pstr Source )  {
+pstr CreateCopy ( pstr & Dest, cpstr Source )  {
   if (Dest)  delete[] Dest;
   if (Source)   {
     Dest = new char[strlen(Source)+1];
@@ -778,7 +753,7 @@ pstr CreateCopy ( pstr & Dest, const pstr Source )  {
   return Dest;
 }
 
-pstr CreateCopy_n ( pstr & Dest, const pstr Source, int n )  {
+pstr CreateCopy_n ( pstr & Dest, cpstr Source, int n )  {
 int l;
   if (Dest)  delete[] Dest;
   if (Source)   {
@@ -792,9 +767,9 @@ int l;
 }
 
 pstr CreateCopCat ( pstr & Dest,
-                    const pstr Source1, const pstr Source2,
-                    const pstr Source3, const pstr Source4,
-                    const pstr Source5 )  {
+                    cpstr Source1, cpstr Source2,
+                    cpstr Source3, cpstr Source4,
+                    cpstr Source5 )  {
   if (Dest) {
     delete[] Dest;
     Dest = NULL;
@@ -803,8 +778,8 @@ pstr CreateCopCat ( pstr & Dest,
 }
 
 pstr CreateCopCat ( pstr & Dest,
-                    const pstr Source1, const pstr Source2,
-                    const pstr Source3, const pstr Source4 )  {
+                    cpstr Source1, cpstr Source2,
+                    cpstr Source3, cpstr Source4 )  {
   if (Dest) {
     delete[] Dest;
     Dest = NULL;
@@ -813,8 +788,8 @@ pstr CreateCopCat ( pstr & Dest,
 }
 
 pstr CreateCopCat ( pstr & Dest,
-                    const pstr Source1, const pstr Source2,
-                    const pstr Source3 )  {
+                    cpstr Source1, cpstr Source2,
+                    cpstr Source3 )  {
   if (Dest) {
     delete[] Dest;
     Dest = NULL;
@@ -823,7 +798,7 @@ pstr CreateCopCat ( pstr & Dest,
 }
 
 pstr CreateCopCat ( pstr & Dest,
-                    const pstr Source1, const pstr Source2 )  {
+                    cpstr Source1, cpstr Source2 )  {
   if (Dest) {
     delete[] Dest;
     Dest = NULL;
@@ -833,9 +808,9 @@ pstr CreateCopCat ( pstr & Dest,
 
 
 pstr CreateConcat ( pstr & Dest,
-                    const pstr Source1, const pstr Source2,
-                    const pstr Source3, const pstr Source4,
-                    const pstr Source5 )  {
+                    cpstr Source1, cpstr Source2,
+                    cpstr Source3, cpstr Source4,
+                    cpstr Source5 )  {
 pstr S;
 int  ld,ls;
   if (Dest) ld = strlen(Dest);
@@ -865,8 +840,8 @@ int  ld,ls;
 
 
 pstr CreateConcat ( pstr & Dest,
-                    const pstr Source1, const pstr Source2,
-                    const pstr Source3, const pstr Source4 )  {
+                    cpstr Source1, cpstr Source2,
+                    cpstr Source3, cpstr Source4 )  {
 pstr S;
 int  ld,ls;
   if (Dest) ld = strlen(Dest);
@@ -894,8 +869,8 @@ int  ld,ls;
 
 
 pstr CreateConcat ( pstr & Dest,
-                    const pstr Source1, const pstr Source2,
-                    const pstr Source3 )  {
+                    cpstr Source1, cpstr Source2,
+                    cpstr Source3 )  {
 pstr S;
 int  ld,ls;
   if (Dest) ld = strlen(Dest);
@@ -920,7 +895,7 @@ int  ld,ls;
 }
 
 pstr CreateConcat ( pstr & Dest,
-                    const pstr Source1, const pstr Source2 )  {
+                    cpstr Source1, cpstr Source2 )  {
 pstr S;
 int  ld,ls;
   if (Dest) ld = strlen(Dest);
@@ -942,7 +917,7 @@ int  ld,ls;
   return Dest;
 }
 
-pstr CreateConcat ( pstr & Dest, const pstr Source )  {
+pstr CreateConcat ( pstr & Dest, cpstr Source )  {
 pstr S;
 int  ld,ls;
   if (Dest)   ld = strlen(Dest);
@@ -963,8 +938,8 @@ int  ld,ls;
 }
 
 
-pstr LastOccurence ( const pstr S, char c )  {
-pstr P=S;
+pstr LastOccurence ( cpstr S, char c )  {
+pstr P=(pstr)S;
 pstr R=NULL;
   while (*P)  {
     if (*P==c)  R = P;
@@ -974,8 +949,8 @@ pstr R=NULL;
 }
 
 
-pstr FirstOccurence ( const pstr S, char c )  {
-pstr P=S;
+pstr FirstOccurence ( cpstr S, char c )  {
+pstr P=(pstr)S;
   while (*P)  {
     if (*P==c)  return P;
     P++;
@@ -983,7 +958,7 @@ pstr P=S;
   return NULL;
 }
 
-int indexOf ( const pstr S, char c )  {
+int indexOf ( cpstr S, char c )  {
 int i=0;
   while (S[i])  {
     if (S[i]==c)  return i;
@@ -992,7 +967,7 @@ int i=0;
   return -1;
 }
 
-pstr FirstOccurence ( const pstr S, int Slen, const pstr Q, int Qlen )  {
+pstr FirstOccurence ( cpstr S, int Slen, cpstr Q, int Qlen )  {
 int i,j,k,l;
   l = Slen-Qlen;
   for (i=0;i<=l;i++)  {
@@ -1001,12 +976,12 @@ int i,j,k,l;
     while (j<Qlen)
       if (S[k++]!=Q[j]) break;
                    else j++;
-    if (j>=Qlen)  return  &(S[i]);
+    if (j>=Qlen)  return  pstr(&(S[i]));
   }
   return NULL;
 }
 
-int indexOf ( const pstr S, int Slen, const pstr Q, int Qlen )  {
+int indexOf ( cpstr S, int Slen, cpstr Q, int Qlen )  {
 int i,j,k,l;
   l = Slen-Qlen;
   for (i=0;i<=l;i++)  {
@@ -1040,7 +1015,7 @@ pstr p=s;
 }
 
 
-void GetString ( pstr L, const pstr S, int M )  {
+void GetString ( pstr L, cpstr S, int M )  {
 //  Copies first M characters of string S into string L,
 // appending the terminating null. If S contains less
 // then M characters, L will be padded with spaces.
@@ -1055,7 +1030,7 @@ int i,j;
 }
 
 
-void GetStrTer ( pstr L, const pstr S, int n, int LMax, int SMax )  {
+void GetStrTer ( pstr L, cpstr S, int n, int LMax, int SMax )  {
 //   Copies at least n (or LMax if LMax<n) first symbols of
 // string S into string L, then continues copying until first
 // space or terminating null is found. If the terminating null
@@ -1065,29 +1040,64 @@ void GetStrTer ( pstr L, const pstr S, int n, int LMax, int SMax )  {
 //   SMax are buffer lengths of L and S, respectively. Even if
 // no space is found, the last character in L will be the
 // terminating null.
-int i,lm1,msl,mnsl;
+int i,k,lm1,msl,mnsl;
   lm1  = LMax-1;
   msl  = IMin(lm1,SMax);
   mnsl = IMin(n,msl);
-  i    = 0;
-  while (S[i] && (i<mnsl))  {
-    L[i] = S[i];
-    i++;
-  }
-  if (!S[i] || (i>=SMax))  {
+  k    = 0;
+  for (i=0;i<mnsl;i++)
+    if (S[i])  L[k++] = S[i];
+         else  break;
+  if ((k>=SMax) || (!S[k]))  {
     lm1 = IMin(n,lm1);
-    while (i<lm1)
-      L[i++] = ' ';
-  } else
-    while (S[i] && (S[i]!=' ') && (i<msl))  {
-      L[i] = S[i];
-      i++;
-    }
-  L[i] = char(0);
+    while (k<lm1)
+      L[k++] = ' ';
+  } else  {
+    lm1 = k;
+    for (i=lm1;i<msl;i++)
+      if (S[i] && (S[i]!=' '))  L[k++] = S[i];
+                          else  break;
+  }
+  L[k] = char(0);
 }
 
 
-void  strcpy_n ( pstr d, const pstr s, int n )  {
+void GetStrTerWin32File ( pstr L, cpstr S, int n, int LMax,
+                          int SMax )  {
+//
+// Version of GetStrTer(..) allowing for spaces in the string.
+//
+//   Copies at least n (or LMax if LMax<n) first symbols of
+// string S into string L, then continues copying until first
+// terminating null is found. If the terminating null
+// is met among the first n characters or if SMax<n, the string
+// L will be padded with spaces till the length of minimum of
+// n and LMax and then terminated with the null. 
+//   SMax are buffer lengths of L and S, respectively. The last
+// character in L will be the terminating null.
+//
+int i,k,lm1,msl,mnsl;
+  lm1  = LMax-1;
+  msl  = IMin(lm1,SMax);
+  mnsl = IMin(n,msl);
+  k    = 0;
+  for (i=0;i<mnsl;i++)
+    if (S[i])  L[k++] = S[i];
+         else  break;
+  if ((!S[k]) || (k>=SMax))  {
+    lm1 = IMin(n,lm1);
+    while (k<lm1)
+      L[k++] = ' ';
+  } else  {
+    lm1 = k;
+    for (i=lm1;i<msl;i++)
+      if (S[i])  L[k++] = S[i];
+           else  break;
+  }
+  L[k] = char(0);
+}
+
+void  strcpy_n ( pstr d, cpstr s, int n )  {
 //   Copies at most n symbols from string s to d, but
 // no more than strlen(s) (s must contain a terminating
 // null). The terminating null IS NEITHER appended NOR
@@ -1101,7 +1111,7 @@ int i;
 }
 
 
-void  strcpy_n1 ( pstr d, const pstr s, int n )  {
+void  strcpy_n1 ( pstr d, cpstr s, int n )  {
 //   Copies at most n last symbols from string s to d, but
 // no more than strlen(s) (s must contain a terminating null).
 // The string in d is aligned to the right and added with
@@ -1116,7 +1126,7 @@ int i,k;
     d[i--] = ' ';
 }
 
-void  strcpy_nr ( pstr d, const pstr s, int n )  {
+void  strcpy_nr ( pstr d, cpstr s, int n )  {
 //   Copies at most n symbols from string s to d, but
 // no more than strlen(s) (s must contain a terminating null).
 // The string in d is aligned to the right and added with
@@ -1132,7 +1142,7 @@ int i,k;
 }
 
 
-void strcpy_ns ( pstr d, const pstr s, int n )  {
+void strcpy_ns ( pstr d, cpstr s, int n )  {
 //   Copies at most n symbols from string s to d, but
 // no more than strlen(s) (s must contain a terminating
 // null). The terminating null IS NEITHER appended NOR
@@ -1149,7 +1159,7 @@ int i;
 }
 
 
-pstr strcpy_cs ( pstr d, const pstr s )  {
+pstr strcpy_cs ( pstr d, cpstr s )  {
 //   Copies string s to string d cutting all spaces at
 // at the end. Thus, " abcde   " will be copied like
 // " abcde" (terminating null appended).
@@ -1168,7 +1178,7 @@ int i;
 }
 
 
-pstr strcpy_ncs ( pstr d, const pstr s, int n )  {
+pstr strcpy_ncs ( pstr d, cpstr s, int n )  {
 //   Copies at most n characters from string s to string d
 // cutting all spaces at at the end. Thus, " abcde   " will
 // be copied like " abc" at n=4 and like " abcde" at n>5
@@ -1187,7 +1197,7 @@ int i;
   return d;
 }
 
-pstr strcpy_css ( pstr d, const pstr s )  {
+pstr strcpy_css ( pstr d, cpstr s )  {
 //   Copies string s to string d cutting all spaces at
 // at the begining and at the end. Thus, " ab c de   "
 // will be copied like "ab c de" (terminating null
@@ -1209,7 +1219,7 @@ int i,k;
   return d;
 }
 
-pstr strcpy_ncss ( pstr d, const pstr s, int n )  {
+pstr strcpy_ncss ( pstr d, cpstr s, int n )  {
 //   Copies at most n characters from string s to string d cutting
 // all spaces at the begining and at the end. Thus, " ab c de  "
 // will be copied like "ab" at n=3 (terminating null appended).
@@ -1231,7 +1241,7 @@ int i,k;
 }
 
 
-pstr strcpy_n0 ( pstr d, const pstr s, int n )  {
+pstr strcpy_n0 ( pstr d, cpstr s, int n )  {
 //   Copies at most n symbols from string s to d, but
 // no more than strlen(s) (s must contain a terminating
 // null). The terminating null IS appended to d.
@@ -1247,7 +1257,7 @@ int i;
 }
 
 
-int strlen_des ( const pstr s )  {
+int strlen_des ( cpstr s )  {
 //  strlen_des returns the length of a string as if all extra
 //  spaces from the latter have been deleted. Extra spaces
 //  include all leading and tracing spaces and any sequential
@@ -1264,7 +1274,7 @@ int i,l;
   return l;
 }
 
-pstr strcpy_des ( pstr d, const pstr s )  {
+pstr strcpy_des ( pstr d, cpstr s )  {
 //  strcpy_des copies string s into string d removing all extra
 //  spaces from the latter. Extra spaces include all leading and
 //  tracing spaces and any sequential spaces when more than one.
@@ -1281,7 +1291,7 @@ int i,j;
   return d;
 }
 
-pstr strcat_des ( pstr d, const pstr s )  {
+pstr strcat_des ( pstr d, cpstr s )  {
 //  strcpy_des appends string s to string d removing all extra
 //  spaces from the latter. Extra spaces include all leading and
 //  tracing spaces and any sequential spaces when more than one.
@@ -1736,47 +1746,47 @@ void mem_write ( Boolean  B, pstr S, int & l )  {
 }
 
 
-void mem_read ( int & I, const pstr S, int & l )  {
+void mem_read ( int & I, cpstr S, int & l )  {
 intUniBin iUB;
   memcpy ( iUB,&(S[l]),sizeof(intUniBin) );
   l += sizeof(intUniBin);
   UniBin2int ( iUB,I );
 }
 
-void mem_read ( short & I, const pstr S, int & l )  {
+void mem_read ( short & I, cpstr S, int & l )  {
 shortUniBin sUB;
   memcpy ( sUB,&(S[l]),sizeof(shortUniBin) );
   l += sizeof(shortUniBin);
   UniBin2short ( sUB,I );
 }
 
-void mem_read ( long & I, const pstr S, int & l )  {
+void mem_read ( long & I, cpstr S, int & l )  {
 longUniBin lUB;
   memcpy ( lUB,&(S[l]),sizeof(longUniBin) );
   l += sizeof(longUniBin);
   UniBin2long ( lUB,I );
 }
 
-void mem_read ( word & W, const pstr S, int & l )  {
+void mem_read ( word & W, cpstr S, int & l )  {
 wordUniBin wUB;
   memcpy ( wUB,&(S[l]),sizeof(wordUniBin) );
   l += sizeof(wordUniBin);
   UniBin2word ( wUB,W );
 }
 
-void mem_read ( realtype & R, const pstr S, int & l )  {
+void mem_read ( realtype & R, cpstr S, int & l )  {
 realUniBin rUB;
   memcpy ( rUB,&(S[l]),sizeof(realUniBin) );
   l += sizeof(realUniBin);
   UniBin2real ( rUB,R );
 }
 
-void mem_read ( pstr L, int len, const pstr S, int & l )  {
+void mem_read ( pstr L, int len, cpstr S, int & l )  {
   memcpy ( L,&(S[l]),len );
   l += len;
 }
 
-void mem_read ( pstr & L, const pstr S, int & l )  {
+void mem_read ( pstr & L, cpstr S, int & l )  {
 int len;
   if (L)  {
     delete[] L;
@@ -1791,7 +1801,7 @@ int len;
   }
 }
 
-void mem_read ( Boolean & B, const pstr S, int & l )  {
+void mem_read ( Boolean & B, cpstr S, int & l )  {
   B = (S[l]=='Y');
   l++;
 }
