@@ -123,14 +123,14 @@ typedef   byte floatUniBin[5];
 #define   False             Boolean(0)
 
 #define   MaxInt            32767
-#define   MinInt            -32768
+#define   MinInt            (-32768)
 #define   MaxWord           65535L
 #define   MaxInt4           2147483647
 
 //    MinInt4 would have to be defined as  -2147483648,
 // however some compilers do not like that. To be on safe,
 // we define it as -2147483647:
-#define   MinInt4           -2147483647
+#define   MinInt4           (-2147483647)
 #define   MaxWord4          4294967295
 
 #define   Pi                3.141592653589793238462643
@@ -463,7 +463,7 @@ extern void strcpy_nr ( pstr d, cpstr s, int n );
 extern void strcpy_ns ( pstr d, cpstr s, int n );
 
 //   strcpy_cs(..) copies string s to string d cutting all
-// spaces at at the end. Thus, " abcde   " will be copied
+// spaces at the end. Thus, " abcde   " will be copied
 // like " abcde" (terminating null appended).
 //   The function returns d.
 extern pstr strcpy_cs ( pstr d, cpstr s );
