@@ -1,53 +1,28 @@
-//  $Id: mmdb_align.cpp,v 1.7 2005/12/19 16:51:28 keb Exp $
+//  $Id: mmdb_align.cpp,v 1.10 2008/07/08 15:28:15 keb Exp $
 //  =================================================================
 //
 //   CCP4 Coordinate Library: support of coordinate-related
 //   functionality in protein crystallography applications.
 //
-//   Copyright (C) Eugene Krissinel 2004.
+//   Copyright (C) Eugene Krissinel 2000-2008.
 //
-//   This library is free software and is distributed under the terms
-//   and conditions of the CCP4 licence agreement as `Part 0' (Annex 2)
-//   software, which is version 2.1 of the GNU Lesser General Public
-//   Licence (LGPL) with the following additional clause:
+//    This library is free software: you can redistribute it and/or 
+//    modify it under the terms of the GNU Lesser General Public 
+//    License version 3, modified in accordance with the provisions 
+//    of the license to address the requirements of UK law.
 //
-//      `You may also combine or link a "work that uses the Library"
-//      to produce a work containing portions of the Library, and
-//      distribute that work under terms of your choice, provided that
-//      you give prominent notice with each copy of the work that the
-//      specified version of the Library is used in it, and that you
-//      include or provide public access to the complete corresponding
-//      machine-readable source code for the Library including whatever
-//      changes were used in the work. (i.e. If you make changes to the
-//      Library you must distribute those, but you do not need to
-//      distribute source or object code to those portions of the work
-//      not covered by this licence.)'
+//    You should have received a copy of the modified GNU Lesser 
+//    General Public License along with this library. If not, copies 
+//    may be downloaded from http://www.ccp4.ac.uk/ccp4license.php
 //
-//   Note that this clause grants an additional right and does not
-//   impose any additional restriction, and so does not affect
-//   compatibility with the GNU General Public Licence (GPL). If you
-//   wish to negotiate other terms, please contact the maintainer.
-//
-//   You can redistribute it and/or modify the library under the terms
-//   of the GNU Lesser General Public License as published by the Free
-//   Software Foundation; either version 2.1 of the License, or (at
-//   your option) any later version.
-//
-//   This library is distributed in the hope that it will be useful,
-//   but WITHOUT ANY WARRANTY; without even the implied warranty of
-//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-//   Lesser General Public License for more details.
-//
-//   You should have received a copy of the CCP4 licence and/or GNU
-//   Lesser General Public License along with this library; if not,
-//   write to the CCP4 Secretary, Daresbury Laboratory, Warrington
-//   WA4 4AD, UK. The GNU Lesser General Public can also be obtained
-//   by writing to the Free Software Foundation, Inc., 59 Temple Place,
-//   Suite 330, Boston, MA 02111-1307 USA
+//    This program is distributed in the hope that it will be useful,
+//    but WITHOUT ANY WARRANTY; without even the implied warranty of
+//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+//    GNU Lesser General Public License for more details.
 //
 //  =================================================================
 //
-//    18.11.05   <--  Date of Last Modification.
+//    08.07.08   <--  Date of Last Modification.
 //                   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //  -----------------------------------------------------------------
 //
@@ -56,7 +31,7 @@
 //  **** Classes    :  CAlignment  ( alignment of character strings )
 //       ~~~~~~~~~~~~  CAlignment1 ( alignment of integer vectors   )
 //
-//  (C) E.Krissinel'  2000-2005
+//  (C) E.Krissinel'  2000-2008
 //
 //  =================================================================
 //
@@ -168,7 +143,7 @@ void  CAlignment::SetScores ( realtype SEqual, realtype SNEqual )  {
   SNEq = SNEqual;
 }
 
-void  CAlignment::Align  ( const pstr S, const pstr T, int Method )  {
+void  CAlignment::Align  ( cpstr S, cpstr T, int Method )  {
 int i,j,i0,j0;
 
   FreeMemory();
@@ -243,7 +218,7 @@ int i,j,i0,j0;
 }
 
 
-void  CAlignment::BuildGATable ( const pstr S, const pstr T,
+void  CAlignment::BuildGATable ( cpstr S, cpstr T,
                                  Boolean FreeSEnd,
                                  Boolean FreeTEnd )  {
 int      i,j;
@@ -304,7 +279,7 @@ realtype V1;
 }
 
 
-void  CAlignment::BuildLATable ( const pstr S, const pstr T )  {
+void  CAlignment::BuildLATable ( cpstr S, cpstr T )  {
 int      i,j;
 realtype V1;
 
@@ -345,7 +320,7 @@ realtype V1;
 
 }
 
-void  CAlignment::PrintVT ( const pstr S, const pstr T )  {
+void  CAlignment::PrintVT ( cpstr S, cpstr T )  {
 int i,j;
   printf ( "\n       " );
   for (j=0;j<=SLen;j++)
@@ -365,7 +340,7 @@ int i,j;
 }
 
 
-void  CAlignment::Backtrace ( const pstr S, const pstr T, int J, int I,
+void  CAlignment::Backtrace ( cpstr S, cpstr T, int J, int I,
                               Boolean StopAtZero )  {
 int       i,j,k, i1,j1, sk,tk;
 char      C;
@@ -373,9 +348,13 @@ realtype  V,SV,TV;
 Boolean   Stop;
 
   //  1. Allocate memory
+
+  if (AlgnS)  delete[] AlgnS;
+  if (AlgnT)  delete[] AlgnT;
+
   i = SLen+TLen+1;
-  if (!AlgnS)  AlgnS = new char[i];
-  if (!AlgnT)  AlgnT = new char[i];
+  AlgnS = new char[i];
+  AlgnT = new char[i];
   memset ( AlgnS,Space,i );
   memset ( AlgnT,Space,i );
 
@@ -497,8 +476,7 @@ Boolean   Stop;
 }
 
 
-void  CAlignment::AdjustEnds ( const pstr S, const pstr T,
-                               int J, int I )  {
+void  CAlignment::AdjustEnds ( cpstr S, cpstr T, int J, int I )  {
 int si,ti,m;
 
   if (J<SLen)  strcat ( AlgnS,&(S[J]) );
@@ -545,8 +523,7 @@ realtype CAlignment::Score ( char A, char B )  {
 
 #define  WrapPeriod  61
 
-void  CAlignment::OutputResults ( RCFile f, const pstr S,
-                                  const pstr T )  {
+void  CAlignment::OutputResults ( RCFile f, cpstr S, cpstr T )  {
 int   k,l,n;
 char  P[3];
 
@@ -903,11 +880,12 @@ realtype  V,SV,TV;
 Boolean   Stop;
 
   //  1. Allocate memory
+
   FreeVectorMemory ( AlgnS,0 );
   FreeVectorMemory ( AlgnT,0 );
   AlgnLen = 0;
 
-  k = SLen+TLen;
+  k = SLen+TLen+1;
   GetVectorMemory  ( AlgnS,k,0 );
   GetVectorMemory  ( AlgnT,k,0 );
   for (i=0;i<k;i++)  {
@@ -1135,9 +1113,11 @@ char  P[10];
     f.WriteLine ( pstr(" End gaps in T-string were not penalized") );
     f.LF();
   }
-  f.WriteParameter ( pstr(" Highest score achieved:"),VAchieved,26,10 );
+  f.WriteParameter ( pstr(" Highest score achieved:"),
+                     VAchieved,26,10 );
   f.LF();
-  f.WriteLine ( pstr(" Aligned S (upper string) and T (lower string):") );
+  f.WriteLine ( pstr(" Aligned S (upper string) and T "
+                     "(lower string):") );
   f.LF();
   k = 0;
   n = 0;
