@@ -22,7 +22,7 @@
 //
 //  =================================================================
 //
-//    08.10.04   <--  Date of Last Modification.
+//    04.02.09   <--  Date of Last Modification.
 //                   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //  -----------------------------------------------------------------
 //
@@ -33,7 +33,7 @@
 //  **** Classes :  CAtom    ( atom class    )
 //       ~~~~~~~~~  CResidue ( residue class )
 //
-//  Copyright (C) E. Krissinel 2000-2008
+//  Copyright (C) E. Krissinel 2000-2009
 //
 //  =================================================================
 //
@@ -941,6 +941,14 @@ realtype dx,dy,dz;
   return  dx*dx + dy*dy + dz*dz;
 }
 
+realtype CAtom::GetDist2 ( PCAtom a, mat33 & r, vect3 & t )  {
+realtype dx,dy,dz;
+  dx = r[0][0]*a->x + r[0][1]*a->y + r[0][2]*a->z + t[0] - x;
+  dy = r[1][0]*a->x + r[1][1]*a->y + r[1][2]*a->z + t[1] - y;
+  dz = r[2][0]*a->x + r[2][1]*a->y + r[2][2]*a->z + t[2] - z;
+  return  dx*dx + dy*dy + dz*dz;
+}
+
 realtype CAtom::GetDist2 ( realtype ax, realtype ay, realtype az )  {
 realtype dx,dy,dz;
   dx = ax - x;
@@ -1903,6 +1911,12 @@ int  CResidue::GetResidueNo()  {
         else  return  -1;
 }
 
+void CResidue::SetChainID ( const ChainID chID )  {
+  if (chain)
+    chain->SetChainID ( chID );
+}
+
+
 int  CResidue::GetCenter ( realtype & x, realtype & y,
                            realtype & z )  {
 int i,k;
@@ -2447,6 +2461,10 @@ Boolean CResidue::isNucleotide()  {
   return ::isNucleotide ( name );
 }
 
+int CResidue::isDNARNA()  {
+  return ::isDNARNA ( name );
+}
+
 Boolean CResidue::isSugar()  {
   return ::isSugar ( name );
 }
