@@ -22,7 +22,7 @@
 //
 //  =================================================================
 //
-//    08.07.08   <--  Date of Last Modification.
+//    04.02.09   <--  Date of Last Modification.
 //                   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //  -----------------------------------------------------------------
 //
@@ -35,7 +35,7 @@
 //  **** Functions :  BondAngle
 //       ~~~~~~~~~~~
 //
-//  Copyright (C) E. Krissinel 2000-2008
+//  Copyright (C) E. Krissinel 2000-2009
 //
 //  =================================================================
 //
@@ -290,6 +290,7 @@ class CAtom : public CUDData  {
     void  CalcAtomStatistics ( RSAtomStat AS );
     realtype GetDist2 ( PCAtom a );
     realtype GetDist2 ( PCAtom a, mat44 & tm );  // tm applies to A
+    realtype GetDist2 ( PCAtom a, mat33 & r, vect3 & t );// tm applies to A
     realtype GetDist2 ( realtype ax, realtype ay, realtype az );
 
     // GetCosine(a1,a2) calculates cosine of angle a1-this-a2,
@@ -467,6 +468,7 @@ class CResidue : public CUDData  {
     void  SetChain ( PCChain Chain_Owner );
     void  SetResID ( const ResName resName, int sqNum,
                      const InsCode ins );
+    void  SetChainID ( const ChainID chID );
 
     void  PDBASCIIAtomDump ( RCFile f        );
     void  MakeAtomCIF      ( PCMMCIFData CIF );
@@ -527,6 +529,7 @@ class CResidue : public CUDData  {
 
     Boolean isAminoacid   ();
     Boolean isNucleotide  ();
+    int     isDNARNA      (); // 0(neither),1(DNA),2(RNA)
     Boolean isSugar       ();
     Boolean isSolvent     ();
     Boolean isModRes      ();
