@@ -1,53 +1,28 @@
-//  $Id: mmdb_chain.cpp,v 1.11 2004/05/17 16:38:13 keb Exp $
+//  $Id: mmdb_chain.cpp,v 1.16 2008/07/08 15:28:15 keb Exp $
 //  =================================================================
 //
 //   CCP4 Coordinate Library: support of coordinate-related
 //   functionality in protein crystallography applications.
 //
-//   Copyright (C) Eugene Krissinel 2004.
+//   Copyright (C) Eugene Krissinel 2000-2008.
 //
-//   This library is free software and is distributed under the terms
-//   and conditions of the CCP4 licence agreement as `Part 0' (Annex 2)
-//   software, which is version 2.1 of the GNU Lesser General Public
-//   Licence (LGPL) with the following additional clause:
+//    This library is free software: you can redistribute it and/or 
+//    modify it under the terms of the GNU Lesser General Public 
+//    License version 3, modified in accordance with the provisions 
+//    of the license to address the requirements of UK law.
 //
-//      `You may also combine or link a "work that uses the Library"
-//      to produce a work containing portions of the Library, and
-//      distribute that work under terms of your choice, provided that
-//      you give prominent notice with each copy of the work that the
-//      specified version of the Library is used in it, and that you
-//      include or provide public access to the complete corresponding
-//      machine-readable source code for the Library including whatever
-//      changes were used in the work. (i.e. If you make changes to the
-//      Library you must distribute those, but you do not need to
-//      distribute source or object code to those portions of the work
-//      not covered by this licence.)'
+//    You should have received a copy of the modified GNU Lesser 
+//    General Public License along with this library. If not, copies 
+//    may be downloaded from http://www.ccp4.ac.uk/ccp4license.php
 //
-//   Note that this clause grants an additional right and does not
-//   impose any additional restriction, and so does not affect
-//   compatibility with the GNU General Public Licence (GPL). If you
-//   wish to negotiate other terms, please contact the maintainer.
-//
-//   You can redistribute it and/or modify the library under the terms
-//   of the GNU Lesser General Public License as published by the Free
-//   Software Foundation; either version 2.1 of the License, or (at
-//   your option) any later version.
-//
-//   This library is distributed in the hope that it will be useful,
-//   but WITHOUT ANY WARRANTY; without even the implied warranty of
-//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-//   Lesser General Public License for more details.
-//
-//   You should have received a copy of the CCP4 licence and/or GNU
-//   Lesser General Public License along with this library; if not,
-//   write to the CCP4 Secretary, Daresbury Laboratory, Warrington
-//   WA4 4AD, UK. The GNU Lesser General Public can also be obtained
-//   by writing to the Free Software Foundation, Inc., 59 Temple Place,
-//   Suite 330, Boston, MA 02111-1307 USA
+//    This program is distributed in the hope that it will be useful,
+//    but WITHOUT ANY WARRANTY; without even the implied warranty of
+//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+//    GNU Lesser General Public License for more details.
 //
 //  =================================================================
 //
-//    17.05.04   <--  Date of Last Modification.
+//    08.07.08   <--  Date of Last Modification.
 //                   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //  -----------------------------------------------------------------
 //
@@ -55,16 +30,16 @@
 //       ~~~~~~~~~
 //  **** Project :  MacroMolecular Data Base (MMDB)
 //       ~~~~~~~~~ 
-//  **** Classes :  CChainContainer ( container for in-chain classes   )
-//       ~~~~~~~~~  CContainerChain ( chain containered class template )
-//                  CDBReference    ( DBREF  records                   )
-//                  CSeqAdv         ( SEQADV records                   )
-//                  CSeqRes         ( SEQRES records                   )
-//                  CModRes         ( MODRES records                   )
-//                  CHetRec         ( HET    records                   )
-//                  CChain          ( MMDB chain class                 )
+//  **** Classes :  CChainContainer ( container for in-chain classes  )
+//       ~~~~~~~~~  CContainerChain ( chain containered class template)
+//                  CDBReference    ( DBREF  records                  )
+//                  CSeqAdv         ( SEQADV records                  )
+//                  CSeqRes         ( SEQRES records                  )
+//                  CModRes         ( MODRES records                  )
+//                  CHetRec         ( HET    records                  )
+//                  CChain          ( MMDB chain class                )
 //
-//  Copyright (C) E. Krissinel 2000-2004
+//  Copyright (C) E. Krissinel 2000-2008
 //
 //  =================================================================
 //
@@ -179,13 +154,14 @@ CDBReference::CDBReference( PCChain Chain_Owner )
   InitDBReference();
 }
 
-CDBReference::CDBReference ( PCChain Chain_Owner, const pstr S )
+CDBReference::CDBReference ( PCChain Chain_Owner, cpstr S )
             : CContainerChain(Chain_Owner)  {
   InitDBReference();
   ConvertPDBASCII ( S );
 }
 
-CDBReference::CDBReference ( RPCStream Object ) : CContainerChain(Object)  {
+CDBReference::CDBReference ( RPCStream Object )
+            : CContainerChain(Object)  {
   InitDBReference();
 }
 
@@ -346,7 +322,7 @@ int           RC,CIFMode,ref_id1,ref_id2;
 }
 
 
-int CDBReference::ConvertPDBASCII ( const pstr S )  {
+int CDBReference::ConvertPDBASCII ( cpstr S )  {
 IDCode idCode;
   if (Chain->chainID[0])  {
     if (S[12]!=Chain->chainID[0])
@@ -439,7 +415,7 @@ CSeqAdv::CSeqAdv ( PCChain Chain_Owner )
   InitSeqAdv();
 }
 
-CSeqAdv::CSeqAdv ( PCChain Chain_Owner, const pstr S )
+CSeqAdv::CSeqAdv ( PCChain Chain_Owner, cpstr S )
        : CContainerChain(Chain_Owner)  {
   InitSeqAdv();
   ConvertPDBASCII ( S );
@@ -481,7 +457,7 @@ void  CSeqAdv::PDBASCIIDump ( pstr S, int N )  {
   strcpy_n   ( &(S[49]),conflict,IMin(strlen(conflict),21) );
 }
 
-int CSeqAdv::ConvertPDBASCII ( const pstr S )  {
+int CSeqAdv::ConvertPDBASCII ( cpstr S )  {
 IDCode idCode;
   if (Chain->chainID[0])  {
     if (S[16]!=Chain->chainID[0])
@@ -719,7 +695,7 @@ int  i,k,sN;
   }
 }
 
-int CSeqRes::ConvertPDBASCII ( const pstr S )  {
+int CSeqRes::ConvertPDBASCII ( cpstr S )  {
 int i,k,sN,nR;
   if (Chain->chainID[0])  {
     if (S[11]!=Chain->chainID[0])
@@ -919,7 +895,7 @@ CModRes::CModRes ( PCChain Chain_Owner )
   InitModRes();
 }
 
-CModRes::CModRes ( PCChain Chain_Owner, const pstr S )
+CModRes::CModRes ( PCChain Chain_Owner, cpstr S )
        : CContainerChain(Chain_Owner)  {
   InitModRes();
   ConvertPDBASCII ( S );
@@ -955,7 +931,7 @@ void  CModRes::PDBASCIIDump ( pstr S, int N )  {
   strcpy_n   ( &(S[29]),comment,IMin(strlen(comment),41) );
 }
 
-int CModRes::ConvertPDBASCII ( const pstr S )  {
+int CModRes::ConvertPDBASCII ( cpstr S )  {
 IDCode idCode;
   if (Chain->chainID[0])  {
     if (S[16]!=Chain->chainID[0])
@@ -982,6 +958,7 @@ IDCode idCode;
 }
 
 void  CModRes::MakeCIF ( PCMMCIFData CIF, int N )  {
+/*  -- apparently wrong use of _struct_conn, to be revised
 PCMMCIFLoop Loop;
 int         RC;
 
@@ -1008,11 +985,15 @@ int         RC;
   Loop->AddString  ( stdRes             ,True );
   Loop->AddString  ( comment            ,True );
 
+*/
+
 }
 
 void  CModRes::GetCIF ( PCMMCIFData CIF, int & Signal )  {
 //  GetCIF(..) must be always run without reference to Chain,
 //  see CModel::GetCIF(..).
+
+/*  -- apparently wrong use of _struct_conn, to be revised
 PCMMCIFLoop   Loop;
 pstr          F;
 int           l,RC;
@@ -1052,7 +1033,8 @@ int           l,RC;
   CIFGetString ( resName,Loop,CIFTAG_PTNR1_LABEL_COMP_ID,Signal,
                  sizeof(ResName),pstr("UNK") );
 
-  if (CIFGetInteger(seqNum,Loop,CIFTAG_PTNR1_LABEL_SEQ_ID,Signal))  return;
+  if (CIFGetInteger(seqNum,Loop,CIFTAG_PTNR1_LABEL_SEQ_ID,Signal))
+    return;
 
   CIFGetString ( insCode,Loop,CIFTAG_NDB_PTNR1_LABEL_INS_CODE,
                  Signal,sizeof(InsCode),pstr(" ") );
@@ -1069,6 +1051,10 @@ int           l,RC;
 
   Signal++;
 
+*/
+
+  Signal = -1;
+
 }
 
 void  CModRes::Copy ( PCContainerClass ModRes )  {
@@ -1114,7 +1100,7 @@ CHetRec::CHetRec ( PCChain Chain_Owner )
   InitHetRec();
 }
 
-CHetRec::CHetRec ( PCChain Chain_Owner, const pstr S )
+CHetRec::CHetRec ( PCChain Chain_Owner, cpstr S )
        : CContainerChain(Chain_Owner)  {
   InitHetRec();
   ConvertPDBASCII ( S );
@@ -1149,7 +1135,7 @@ void  CHetRec::PDBASCIIDump ( pstr S, int N )  {
   strcpy_n   ( &(S[30]),comment,IMin(strlen(comment),40) );
 }
 
-int  CHetRec::ConvertPDBASCII ( const pstr S )  {
+int  CHetRec::ConvertPDBASCII ( cpstr S )  {
   if (Chain->chainID[0])  {
     if (S[12]!=Chain->chainID[0])
       return Error_WrongChainID;
@@ -1228,7 +1214,8 @@ int           RC;
   CIFGetString ( insCode,Loop,CIFTAG_INS_CODE,Signal,sizeof(InsCode),
                  pstr(" ") );
 
-  if (CIFGetInteger(numHetAtoms,Loop,CIFTAG_NUMBER_ATOMS_NH,Signal))  return;
+  if (CIFGetInteger(numHetAtoms,Loop,CIFTAG_NUMBER_ATOMS_NH,Signal))
+    return;
 
   F = Loop->GetString ( CIFTAG_DETAILS,Signal,RC );
   if ((!RC) && F)  {
@@ -1357,7 +1344,7 @@ int i;
         Residue[i]->CheckInAtoms();
 }
 
-int CChain::ConvertDBREF ( const pstr PDBString ) {
+int CChain::ConvertDBREF ( cpstr PDBString ) {
 int              RC;
 PCContainerChain ContainerChain;
   ContainerChain = new CDBReference(this);
@@ -1370,7 +1357,7 @@ PCContainerChain ContainerChain;
   return 0;
 }
     
-int CChain::ConvertSEQADV ( const pstr PDBString ) {
+int CChain::ConvertSEQADV ( cpstr PDBString ) {
 int              RC;
 PCContainerChain ContainerChain;
   ContainerChain = new CSeqAdv(this);
@@ -1383,11 +1370,11 @@ PCContainerChain ContainerChain;
   return 0;
 }
 
-int CChain::ConvertSEQRES ( const pstr PDBString ) {
+int CChain::ConvertSEQRES ( cpstr PDBString ) {
   return SeqRes.ConvertPDBASCII ( PDBString );
 }
         
-int  CChain::ConvertMODRES ( const pstr PDBString ) {
+int  CChain::ConvertMODRES ( cpstr PDBString ) {
 int              RC;
 PCContainerChain ContainerChain;
   ContainerChain = new CModRes(this);
@@ -1400,7 +1387,7 @@ PCContainerChain ContainerChain;
   return 0;
 }
         
-int  CChain::ConvertHET ( const pstr PDBString ) {
+int  CChain::ConvertHET ( cpstr PDBString ) {
 int              RC;
 PCContainerChain ContainerChain;
   ContainerChain = new CHetRec(this);
@@ -1475,7 +1462,8 @@ int i;
   } else  {
     for (i=0;i<nResidues;i++)
       if (Residue[i])  {
-        if ((seqNum==Residue[i]->seqNum) && (!Residue[i]->insCode[0]))  {
+        if ((seqNum==Residue[i]->seqNum) &&
+            (!Residue[i]->insCode[0]))  {
           if (!strcmp(resName,Residue[i]->name))
             return Residue[i]; // it is there; just return the pointer
           else if (!Enforce)
@@ -1866,7 +1854,8 @@ PCResidue res;
   return NULL;
 }
 
-PCAtom CChain::GetAtom ( int seqNo, const InsCode insCode, int atomNo )  {
+PCAtom CChain::GetAtom ( int seqNo, const InsCode insCode,
+                         int atomNo )  {
 PCResidue res;
   res = GetResidue ( seqNo,insCode );  
   if (res)  {
@@ -1900,7 +1889,7 @@ PCResidue res;
 }
 
 void CChain::GetAtomTable ( int seqNo, const InsCode insCode,
-                            PPCAtom & atomTable, int & NumberOfAtoms )  {
+                         PPCAtom & atomTable, int & NumberOfAtoms )  {
 PCResidue res;
   atomTable     = NULL;
   NumberOfAtoms = 0;
@@ -2035,7 +2024,8 @@ int i,n;
 }
 
 
-int CChain::AddAtom ( int seqNo, const InsCode insCode, PCAtom atom )  {
+int CChain::AddAtom ( int seqNo, const InsCode insCode,
+                      PCAtom atom )  {
 PCResidue res;
   res = GetResidue ( seqNo,insCode );  
   if (res) return res->AddAtom ( atom );
@@ -2269,7 +2259,7 @@ int  CChain::CheckID ( const ChainID chID )  {
   return 0;
 }
 
-int  CChain::CheckIDS ( const pstr CID )  {
+int  CChain::CheckIDS ( cpstr CID )  {
 ChainID  chn;
 InsCode  inscode;
 ResName  resname;
@@ -2337,7 +2327,7 @@ int  CChain::PutUDData ( int UDDhandle, realtype rudd )  {
   else  return  UDDATA_WrongUDRType;
 }
 
-int  CChain::PutUDData ( int UDDhandle, const pstr sudd )  {
+int  CChain::PutUDData ( int UDDhandle, cpstr sudd )  {
   if (UDDhandle & UDRF_CHAIN)
         return  CUDData::putUDData ( UDDhandle,sudd );
   else  return  UDDATA_WrongUDRType;
