@@ -1,49 +1,24 @@
-//  $Id: mmdb_chain.h,v 1.8 2004/03/26 15:00:58 keb Exp $
+//  $Id: mmdb_chain.h,v 1.12 2008/07/08 15:28:15 keb Exp $
 //  =================================================================
 //
 //   CCP4 Coordinate Library: support of coordinate-related
 //   functionality in protein crystallography applications.
 //
-//   Copyright (C) Eugene Krissinel 2004.
+//   Copyright (C) Eugene Krissinel 2000-2008.
 //
-//   This library is free software and is distributed under the terms
-//   and conditions of the CCP4 licence agreement as `Part 0' (Annex 2)
-//   software, which is version 2.1 of the GNU Lesser General Public
-//   Licence (LGPL) with the following additional clause:
+//    This library is free software: you can redistribute it and/or 
+//    modify it under the terms of the GNU Lesser General Public 
+//    License version 3, modified in accordance with the provisions 
+//    of the license to address the requirements of UK law.
 //
-//      `You may also combine or link a "work that uses the Library"
-//      to produce a work containing portions of the Library, and
-//      distribute that work under terms of your choice, provided that
-//      you give prominent notice with each copy of the work that the
-//      specified version of the Library is used in it, and that you
-//      include or provide public access to the complete corresponding
-//      machine-readable source code for the Library including whatever
-//      changes were used in the work. (i.e. If you make changes to the
-//      Library you must distribute those, but you do not need to
-//      distribute source or object code to those portions of the work
-//      not covered by this licence.)'
+//    You should have received a copy of the modified GNU Lesser 
+//    General Public License along with this library. If not, copies 
+//    may be downloaded from http://www.ccp4.ac.uk/ccp4license.php
 //
-//   Note that this clause grants an additional right and does not
-//   impose any additional restriction, and so does not affect
-//   compatibility with the GNU General Public Licence (GPL). If you
-//   wish to negotiate other terms, please contact the maintainer.
-//
-//   You can redistribute it and/or modify the library under the terms
-//   of the GNU Lesser General Public License as published by the Free
-//   Software Foundation; either version 2.1 of the License, or (at
-//   your option) any later version.
-//
-//   This library is distributed in the hope that it will be useful,
-//   but WITHOUT ANY WARRANTY; without even the implied warranty of
-//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-//   Lesser General Public License for more details.
-//
-//   You should have received a copy of the CCP4 licence and/or GNU
-//   Lesser General Public License along with this library; if not,
-//   write to the CCP4 Secretary, Daresbury Laboratory, Warrington
-//   WA4 4AD, UK. The GNU Lesser General Public can also be obtained
-//   by writing to the Free Software Foundation, Inc., 59 Temple Place,
-//   Suite 330, Boston, MA 02111-1307 USA
+//    This program is distributed in the hope that it will be useful,
+//    but WITHOUT ANY WARRANTY; without even the implied warranty of
+//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+//    GNU Lesser General Public License for more details.
 //
 //  =================================================================
 //
@@ -55,17 +30,17 @@
 //       ~~~~~~~~~
 //  **** Project :  MacroMolecular Data Base (MMDB)
 //       ~~~~~~~~~
-//  **** Classes :  CProModel       ( a virtue of CModel               )
-//       ~~~~~~~~~  CChainContainer ( container of in-chain classes    )
-//                  CContainerChain ( chain containered class template )
-//                  CDBReference    ( DBREF  records                   )
-//                  CSeqAdv         ( SEQADV records                   )
-//                  CSeqRes         ( SEQRES records                   )
-//                  CModRes         ( MODRES records                   )
-//                  CHetRec         ( HET    records                   )
-//                  CChain          ( chain class                      )
+//  **** Classes :  CProModel       ( a virtue of CModel              )
+//       ~~~~~~~~~  CChainContainer ( container of in-chain classes   )
+//                  CContainerChain ( chain containered class template)
+//                  CDBReference    ( DBREF  records                  )
+//                  CSeqAdv         ( SEQADV records                  )
+//                  CSeqRes         ( SEQRES records                  )
+//                  CModRes         ( MODRES records                  )
+//                  CHetRec         ( HET    records                  )
+//                  CChain          ( chain class                     )
 //
-//  Copyright (C) E. Krissinel 2001-2004
+//  Copyright (C) E. Krissinel 2000-2008
 //
 //  =================================================================
 //
@@ -120,8 +95,11 @@ class CProModel : public CUDData  {
 
     //  GetNumberOfModels() returns TOTAL number of models
     virtual int GetNumberOfModels() { return 0;    }
-    //  GetNumberOfAllAtoms() returns TOTAL number of atoms in all models
+
+    //  GetNumberOfAllAtoms() returns TOTAL number of atoms in
+    // all models
     virtual int GetNumberOfAllAtoms() { return 0;    }
+
     //  returns pointer to the general Atom array
     virtual PPCAtom     GetAllAtoms() { return NULL; }
 
@@ -132,7 +110,7 @@ class CProModel : public CUDData  {
 
   protected :
 
-    virtual int  _ExcludeChain   ( const ChainID chainID ) { return 0; } 
+    virtual int  _ExcludeChain ( const ChainID chainID ) { return 0; } 
 
 };
 
@@ -154,12 +132,13 @@ class CChainContainer : public CClassContainer  {
 
     PCContainerClass MakeContainerClass ( int ClassID );
 
-    void  SetChain ( PCChain Chain_Owner );  //  must be set before using
-                                             //  the Container
+    void  SetChain ( PCChain Chain_Owner ); // must be set before using
+                                            // the Container
     
     // special functions used in CModel::GetCIF(..)
     pstr  Get1stChainID ();
-    void  MoveByChainID ( ChainID chainID, PCChainContainer ChainContainer );
+    void  MoveByChainID ( ChainID chainID,
+                          PCChainContainer ChainContainer );
 
   protected :
     PCChain Chain;
@@ -180,7 +159,7 @@ class CContainerChain : public CContainerClass {
 
     CContainerChain ();
     CContainerChain ( PCChain Chain_Owner );
-    CContainerChain ( RPCStream Object    ) : CContainerClass(Object) {}
+    CContainerChain ( RPCStream Object    ) : CContainerClass(Object){}
 
     void SetChain   ( PCChain Chain_Owner );
 
@@ -200,27 +179,27 @@ class CDBReference : public CContainerChain  {
 
   public :
 
-    int      seqBeg;      // initial sequence number of the PDB seq-ce segment
-    InsCode  insBeg;      // initial insertion code of the PDB sequence segm-t
-    int      seqEnd;      // ending sequence number of the PDB sequence segm-t
-    InsCode  insEnd;      // ending insertion code of the PDB sequence segment
+    int      seqBeg;      // initial seq num of the PDB seq-ce segment
+    InsCode  insBeg;      // initial ins code of the PDB seq-ce segm-t
+    int      seqEnd;      // ending seq number of the PDB seq-ce segm-t
+    InsCode  insEnd;      // ending ins code of the PDB seq-ce segment
     DBName   database;    // sequence database name
     DBAcCode dbAccession; // sequence database accession code
     DBIdCode dbIdCode;    // sequence database identification code
-    int      dbseqBeg;    // initial sequence number of the database segment
-    InsCode  dbinsBeg;    // insertion code of initial residue of the segment
-    int      dbseqEnd;    // ending sequence number of the database segment
-    InsCode  dbinsEnd;    // ins-n code of the ending residue of the segment
+    int      dbseqBeg;    // initial seq number of the database segment
+    InsCode  dbinsBeg;    // ins code of initial residue of the segment
+    int      dbseqEnd;    // ending seq number of the database segment
+    InsCode  dbinsEnd;   // ins code of the ending residue of the seg-t
     
     CDBReference ();
     CDBReference ( PCChain Chain_Owner );
-    CDBReference ( PCChain Chain_Owner, const pstr S );
+    CDBReference ( PCChain Chain_Owner, cpstr S );
     CDBReference ( RPCStream Object );
     ~CDBReference();
 
     void  PDBASCIIDump    ( pstr S, int N );
     void  MakeCIF         ( PCMMCIFData CIF, int N );
-    int   ConvertPDBASCII ( const pstr S );
+    int   ConvertPDBASCII ( cpstr S );
     void  GetCIF          ( PCMMCIFData CIF, int & Signal );
     int   GetClassID      () { return ClassID_DBReference; }
 
@@ -236,7 +215,7 @@ class CDBReference : public CContainerChain  {
 };
 
 
-//  ==================  CSeqAdv  ========================
+//  ====================  CSeqAdv  ===========================
 
 DefineClass(CSeqAdv)
 DefineStreamFunctions(CSeqAdv)
@@ -256,12 +235,12 @@ class CSeqAdv : public CContainerChain  {
     
     CSeqAdv ();
     CSeqAdv ( PCChain Chain_Owner );
-    CSeqAdv ( PCChain Chain_Owner, const pstr S );
+    CSeqAdv ( PCChain Chain_Owner, cpstr S );
     CSeqAdv ( RPCStream Object );
     ~CSeqAdv();
 
     void  PDBASCIIDump    ( pstr S, int N );
-    int   ConvertPDBASCII ( const pstr S );
+    int   ConvertPDBASCII ( cpstr S );
 
     void  MakeCIF         ( PCMMCIFData CIF, int N );
     void  GetCIF          ( PCMMCIFData CIF, int & Signal );
@@ -301,7 +280,7 @@ class CSeqRes : public CStream  {
 
     void  SetChain        ( PCChain Chain_Owner );
     void  PDBASCIIDump    ( RCFile f );
-    int   ConvertPDBASCII ( const pstr S );
+    int   ConvertPDBASCII ( cpstr  S );
 
     void  MakeCIF         ( PCMMCIFData CIF );
     int   GetCIF          ( PCMMCIFData CIF );
@@ -339,13 +318,13 @@ class CModRes : public CContainerChain  {
     
     CModRes ();
     CModRes ( PCChain Chain_Owner );
-    CModRes ( PCChain Chain_Owner, const pstr S );
+    CModRes ( PCChain Chain_Owner, cpstr S );
     CModRes ( RPCStream Object );
     ~CModRes();
 
     void  PDBASCIIDump    ( pstr S, int N );
     void  MakeCIF         ( PCMMCIFData CIF, int N );
-    int   ConvertPDBASCII ( const pstr S );
+    int   ConvertPDBASCII ( cpstr S );
     void  GetCIF          ( PCMMCIFData CIF, int & Signal );
     int   GetClassID      () { return ClassID_ModRes; }
 
@@ -379,13 +358,13 @@ class CHetRec : public CContainerChain  {
     
     CHetRec ();
     CHetRec ( PCChain Chain_Owner );
-    CHetRec ( PCChain Chain_Owner, const pstr S );
+    CHetRec ( PCChain Chain_Owner, cpstr S );
     CHetRec ( RPCStream Object );
     ~CHetRec();
 
     void  PDBASCIIDump    ( pstr S, int N );
     void  MakeCIF         ( PCMMCIFData CIF, int N );
-    int   ConvertPDBASCII ( const pstr S );
+    int   ConvertPDBASCII ( cpstr S );
     void  GetCIF          ( PCMMCIFData CIF, int & Signal );
     int   GetClassID      () { return ClassID_Het; }
 
@@ -444,11 +423,11 @@ class CChain : public CUDData  {
     //   ConvertXXXXX(..) functions do not check for record name
     // and assume that PDBString is at least 81 symbols long
     // (including the terminating null).
-    int  ConvertDBREF  ( const pstr PDBString );
-    int  ConvertSEQADV ( const pstr PDBString );
-    int  ConvertSEQRES ( const pstr PDBString );
-    int  ConvertMODRES ( const pstr PDBString );
-    int  ConvertHET    ( const pstr PDBString );
+    int  ConvertDBREF  ( cpstr PDBString );
+    int  ConvertSEQADV ( cpstr PDBString );
+    int  ConvertSEQRES ( cpstr PDBString );
+    int  ConvertMODRES ( cpstr PDBString );
+    int  ConvertHET    ( cpstr PDBString );
 
     // This function should be used for testing purposes only.
     // A full PDB ASCII dump for all models and chains involved
@@ -461,7 +440,7 @@ class CChain : public CUDData  {
 
     //  -----------------  Extracting residues  -------------------------
 
-    int GetNumberOfResidues(); // returns number of residues in the chain
+    int GetNumberOfResidues(); // returns number of res-s in the chain
     PCResidue GetResidue   ( int resNo ); // returns resNo-th residue
                                           // in the chain;
                                           // 0<=resNo<nResidues
@@ -473,12 +452,13 @@ class CChain : public CUDData  {
     PCResidue GetResidue ( int seqNum, const InsCode insCode );
 
     //   GetResidueNo(..) returns the residue number in the chain's
-    // residues table. Residues are numbered as 0..nres-1 as they appear
-    // in the coordinate file.
+    // residues table. Residues are numbered as 0..nres-1 as they
+    // appear in the coordinate file.
     //   If residue is not found, the function returns -1.
     int  GetResidueNo ( int seqNum, const InsCode insCode );
 
-    void GetResidueTable ( PPCResidue & resTable, int & NumberOfResidues );
+    void GetResidueTable ( PPCResidue & resTable,
+                           int & NumberOfResidues );
 
     //   GetResidueCreate(..) returns pointer on residue, whose name,
     // sequence number and insertion code are given by resName, seqNum
@@ -491,37 +471,39 @@ class CChain : public CUDData  {
     // is no guarantee that any function operating on the sequence
     // number and insertion code will work properly.
     PCResidue GetResidueCreate ( const ResName resName, int seqNum,
-                                 const InsCode insCode, Boolean Enforce );
+                             const InsCode insCode, Boolean Enforce );
 
 
-    //  ------------------  Deleting residues  --------------------------
+    //  ------------------  Deleting residues  ----------------------
 
-    int  DeleteResidue ( int resNo ); // returns number of deleted res-s
+    int  DeleteResidue ( int resNo ); // returns num of deleted res-s
     int  DeleteResidue ( int seqNum, const InsCode insCode );
     int  DeleteAllResidues();
     int  DeleteSolvent    ();
-    void TrimResidueTable ();  // do not forget to call after all deletions
+    void TrimResidueTable ();  // do not forget to call after all dels
 
-    //  -------------------  Adding residues  ---------------------------
+    //  -------------------  Adding residues  -----------------------
 
     //   AddResidue(..) adds residue to the chain, InsResidue inserts
-    // the residue on the specified position of the chain (other residues
-    // are shifted up to the end of chain). Position in the chain may be
-    // specified by a serial number (that is position in the residue table)
-    // or by seqNum and insCode of one of the chain's residues (the new
-    // residue is then inserted before that one). If the chain is associated
-    // with a coordinate hierarchy, and residue 'res' is not, the latter
-    // is checked in automatically. If residue 'res' belongs to any
-    // coordinate hierarchy (even though that of the residue), it is
-    // *copied* rather than simply taken over, and is checked in.
-    //   If the chain is not associated with a coordinate hierarchy, all
-    // added residues will be checked in automatically once the chain
-    // is checked in.
+    // the residue on the specified position of the chain (other
+    // residues are shifted up to the end of chain). Position in the
+    // chain may be specified by a serial number (that is position in
+    // the residue table) or by seqNum and insCode of one of the
+    // chain's residues (the new residue is then inserted before that
+    // one). If the chain is associated with a coordinate hierarchy,
+    // and residue 'res' is not, the latter is checked in
+    // automatically. If residue 'res' belongs to any coordinate
+    // hierarchy (even though that of the residue), it is *copied*
+    // rather than simply taken over, and is checked in.
+    //   If the chain is not associated with a coordinate hierarchy,
+    // all added residues will be checked in automatically once the
+    // chain is checked in.
     int  AddResidue ( PCResidue res );
     int  InsResidue ( PCResidue res, int pos );
-    int  InsResidue ( PCResidue res, int seqNum, const InsCode insCode );
+    int  InsResidue ( PCResidue res, int seqNum,
+                      const InsCode insCode );
 
-    //  --------------------  Extracting atoms  -------------------------
+    //  --------------------  Extracting atoms  ---------------------
 
     int  GetNumberOfAtoms ( Boolean countTers );
     int  GetNumberOfAtoms ( int seqNo, const InsCode insCode );
@@ -552,14 +534,15 @@ class CChain : public CUDData  {
     // attempt to deallocate it first.
     //   The application is responsible for deleting atomTable,
     // however it must not touch atom pointers, i.e. use simply
-    // "delete[] atomTable;". Never pass atomTable from GetAtomTable(..)
-    // into this function, unless you set it to NULL before doing that.
+    // "delete[] atomTable;". Never pass atomTable from
+    // GetAtomTable(..) into this function, unless you set it to NULL
+    // before doing that.
     void GetAtomTable1 ( int seqNo, const InsCode insCode,
                          PPCAtom & atomTable, int & NumberOfAtoms );
     void GetAtomTable1 ( int resNo,
                          PPCAtom & atomTable, int & NumberOfAtoms );
 
-    //  ---------------------  Deleting atoms  --------------------------
+    //  ---------------------  Deleting atoms  ----------------------
 
     int DeleteAtom ( int            seqNo,
                      const InsCode  insCode,
@@ -583,20 +566,20 @@ class CChain : public CUDData  {
     // occupancy, if those are equal or unspecified, the one with
     // "least" alternative location indicator.
     //  The function returns the number of deleted. All tables remain
-    // untrimmed, so that explicit trimming or calling FinishStructEdit()
-    // is required.
+    // untrimmed, so that explicit trimming or calling
+    // FinishStructEdit() is required.
     int DeleteAltLocs();
 
-    //  ----------------------  Adding atoms  ---------------------------
+    //  ----------------------  Adding atoms  -----------------------
 
     int AddAtom ( int seqNo, const InsCode insCode, PCAtom atom );
     int AddAtom ( int resNo, PCAtom atom );
 
-    //  -----------------------------------------------------------------
+    //  -------------------------------------------------------------
 
     void  ApplyTransform ( mat44 & TMatrix );  // transforms all
-                                               // coordinates by multiplying
-                                               // with matrix TMatrix
+                                         // coordinates by multiplying
+                                         // with matrix TMatrix
 
     int   GetModelNum();
     PCModel GetModel () { return (PCModel)model; }
@@ -608,7 +591,7 @@ class CChain : public CUDData  {
     void  CalcAtomStatistics ( RSAtomStat AS );
 
     int   CheckID    ( const ChainID chID );
-    int   CheckIDS   ( const pstr    CID  );
+    int   CheckIDS   ( cpstr CID  );
 
     pstr  GetEntryID ();
     void  SetEntryID ( const IDCode idCode );
@@ -633,9 +616,9 @@ class CChain : public CUDData  {
 
 
     // -------  user-defined data handlers
-    int   PutUDData ( int UDDhandle, int        iudd );
-    int   PutUDData ( int UDDhandle, realtype   rudd );
-    int   PutUDData ( int UDDhandle, const pstr sudd );
+    int   PutUDData ( int UDDhandle, int      iudd );
+    int   PutUDData ( int UDDhandle, realtype rudd );
+    int   PutUDData ( int UDDhandle, cpstr    sudd );
 
     int   GetUDData ( int UDDhandle, int      & iudd );
     int   GetUDData ( int UDDhandle, realtype & rudd );
@@ -667,7 +650,8 @@ class CChain : public CUDData  {
 
     void  ExpandResidueArray ( int inc );
     //   _ExcludeResidue(..) excludes (but does not dispose!) a residue
-    // from the chain. Returns 1 if the chain gets empty and 0 otherwise.
+    // from the chain. Returns 1 if the chain gets empty and 0
+    // otherwise.
     int   _ExcludeResidue ( const ResName resName, int seqNum,
                             const InsCode insCode );
     void  _copy ( PCChain Chain );
