@@ -1,49 +1,24 @@
-//  $Id: mmdb_cifdefs.h,v 1.3 2004/01/28 17:22:32 keb Exp $
+//  $Id: mmdb_cifdefs.h,v 1.6 2008/07/08 15:28:15 keb Exp $
 //  =================================================================
 //
 //   CCP4 Coordinate Library: support of coordinate-related
 //   functionality in protein crystallography applications.
 //
-//   Copyright (C) Eugene Krissinel 2004.
+//   Copyright (C) Eugene Krissinel 2000-2008.
 //
-//   This library is free software and is distributed under the terms
-//   and conditions of the CCP4 licence agreement as `Part 0' (Annex 2)
-//   software, which is version 2.1 of the GNU Lesser General Public
-//   Licence (LGPL) with the following additional clause:
+//    This library is free software: you can redistribute it and/or 
+//    modify it under the terms of the GNU Lesser General Public 
+//    License version 3, modified in accordance with the provisions 
+//    of the license to address the requirements of UK law.
 //
-//      `You may also combine or link a "work that uses the Library"
-//      to produce a work containing portions of the Library, and
-//      distribute that work under terms of your choice, provided that
-//      you give prominent notice with each copy of the work that the
-//      specified version of the Library is used in it, and that you
-//      include or provide public access to the complete corresponding
-//      machine-readable source code for the Library including whatever
-//      changes were used in the work. (i.e. If you make changes to the
-//      Library you must distribute those, but you do not need to
-//      distribute source or object code to those portions of the work
-//      not covered by this licence.)'
+//    You should have received a copy of the modified GNU Lesser 
+//    General Public License along with this library. If not, copies 
+//    may be downloaded from http://www.ccp4.ac.uk/ccp4license.php
 //
-//   Note that this clause grants an additional right and does not
-//   impose any additional restriction, and so does not affect
-//   compatibility with the GNU General Public Licence (GPL). If you
-//   wish to negotiate other terms, please contact the maintainer.
-//
-//   You can redistribute it and/or modify the library under the terms
-//   of the GNU Lesser General Public License as published by the Free
-//   Software Foundation; either version 2.1 of the License, or (at
-//   your option) any later version.
-//
-//   This library is distributed in the hope that it will be useful,
-//   but WITHOUT ANY WARRANTY; without even the implied warranty of
-//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-//   Lesser General Public License for more details.
-//
-//   You should have received a copy of the CCP4 licence and/or GNU
-//   Lesser General Public License along with this library; if not,
-//   write to the CCP4 Secretary, Daresbury Laboratory, Warrington
-//   WA4 4AD, UK. The GNU Lesser General Public can also be obtained
-//   by writing to the Free Software Foundation, Inc., 59 Temple Place,
-//   Suite 330, Boston, MA 02111-1307 USA
+//    This program is distributed in the hope that it will be useful,
+//    but WITHOUT ANY WARRANTY; without even the implied warranty of
+//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+//    GNU Lesser General Public License for more details.
 //
 //  =================================================================
 //
@@ -58,7 +33,7 @@
 //
 //      CIF Definitions
 //
-//  E. Krissinel 2000-2002
+//  (C) E. Krissinel 2000-2008
 //
 //  =================================================================
 //
@@ -136,8 +111,8 @@ extern pstr CIFName ( int NameID, int Mode );
 #define CIFCAT_STRUCT_SHEET_HBOND          pstr("_struct_sheet_hbond")
 #define CIFCAT_SYMMETRY                    pstr("_symmetry")
 #define CIFCAT_OBSLTE                      pstr("_ndb_database_pdb_obs_spr")
-                                                                        
 
+                                                                       
 #define CIFTAG_ANGLE_ALPHA                    pstr("angle_alpha")
 #define CIFTAG_ANGLE_BETA                     pstr("angle_beta")
 #define CIFTAG_ANGLE_GAMMA                    pstr("angle_gamma")
@@ -328,6 +303,21 @@ extern pstr CIFName ( int NameID, int Mode );
 #define CIFTAG_U33_ESD                        pstr("u[3][3]_esd")
 #define CIFTAG_Z_PDB                          pstr("z_pdb")
 
+#define CIFTAG_CONN_PTNR1_AUTH_ATOM_ID        pstr("ptnr1_auth_atom_id")
+#define CIFTAG_CONN_PDBX_PTNR1_AUTH_ALT_ID    pstr("pdbx_ptnr1_auth_alt_id")
+#define CIFTAG_CONN_PTNR1_AUTH_COMP_ID        pstr("ptnr1_auth_comp_id")
+#define CIFTAG_CONN_PTNR1_AUTH_ASYM_ID        pstr("ptnr1_auth_asym_id")
+#define CIFTAG_CONN_PTNR1_AUTH_SEQ_ID         pstr("ptnr1_auth_seq_id")
+#define CIFTAG_CONN_PDBX_PTNR1_PDB_INS_CODE   pstr("pdbx_ptnr1_PDB_ins_code")
+#define CIFTAG_CONN_PTNR2_AUTH_ATOM_ID        pstr("ptnr2_auth_atom_id")
+#define CIFTAG_CONN_PDBX_PTNR2_AUTH_ALT_ID    pstr("pdbx_ptnr2_auth_alt_id")
+#define CIFTAG_CONN_PTNR2_AUTH_COMP_ID        pstr("ptnr2_auth_comp_id")
+#define CIFTAG_CONN_PTNR2_AUTH_ASYM_ID        pstr("ptnr2_auth_asym_id")
+#define CIFTAG_CONN_PTNR2_AUTH_SEQ_ID         pstr("ptnr2_auth_seq_id")
+#define CIFTAG_CONN_PDBX_PTNR2_PDB_INS_CODE   pstr("pdbx_ptnr2_PDB_ins_code")
+#define CIFTAG_CONN_PTNR1_SYMMETRY            pstr("ptnr1_symmetry")
+#define CIFTAG_CONN_PTNR2_SYMMETRY            pstr("ptnr2_symmetry")
+
 
 #endif
 
