@@ -467,8 +467,8 @@ class CMMDBCoorManager : public CMMDBFile  {
     //   GetAtomTable1(..) returns atom table without TER atoms and
     // without NULL atom pointers. NumberOfAtoms returns the actual
     // number of atom pointers in atomTable.
-    //   atomTable is allocated withing the function. If it was
-    // not set to NULL before calling the function, the latter will
+    //   atomTable is allocated within the function. If it was
+    // not set to NULL before calling the function, the function will
     // attempt to deallocate it first.
     //   The application is responsible for deleting atomTable,
     // however it must not touch atom pointers, i.e. use simply
@@ -748,12 +748,39 @@ class CMMDBCoorManager : public CMMDBFile  {
                                    // fields. This ID may be useful
                                    // if contacts are obtained in
                                    // multiple calls of the function
-             int     bricking=0    //  bricking control; may be a
+             int     bricking=0,   //  bricking control; may be a
+                                   // combination of BRICK_ON_1 or
+                                   // BRICK_ON_2 with BRICK_READY
+             Boolean doSqrt=True   // if False, then SContact contains
+                                   // square distances
+                       );
+
+    //  Simplified optimized for speed version:
+    //    - no NULL pointers and Ters in AIndex1 and AIndex2
+    //    - no checks for identity atoms in AIndex1 and AIndex2
+    //    - contact must be pre-allocated with at least ilen1*ilen2 elements
+    //    - contact returns square distances
+    //    - ncontacts is always reset
+    void  SeekContacts (
+             PPCAtom    AIndex1,   //  1st atom index [0..ilen1-1]
+             int        ilen1,     //  length of 1st index
+             PPCAtom    AIndex2,   //  2nd atom index [0..ilen2-1] to
+                                   // check for contact with 1st index
+             int        ilen2,     //  length of 2nd index
+             realtype   contDist,  //  maximal contact distance
+             PSContact  contact,   //  indices of contacting atoms
+                                   // [0..ncontacts-1]. contact[i].id1
+                                   // contains number of atom from 1st
+                                   // index, and contact[i].id2
+                                   // contains number of atom from 2nd
+                                   // index, contacting with the former
+                                   // one. Must be pre-allocated
+             int &      ncontacts, //  number of contacts found
+             int       bricking=0  //  bricking control; may be a
                                    // combination of BRICK_ON_1 or
                                    // BRICK_ON_2 with BRICK_READY
                        );
 
-
     void  SeekContacts (
            PPCAtom       AIndex1,  //  1st atom index [0..ilen1-1]
            int           ilen1,    //  length of 1st index
@@ -845,7 +872,7 @@ extern void EulerRotation ( PPCAtom A, int nA,
 //   GetVecRotMatrix(..) calculates the rotation matrix for
 // rotation by angle alpha about arbitrary vector directed
 // as (vx,vy,vz) = (vx2-vx1,vy2-vy1,vz2-vz1).
-extern void  GetVecRotMatrix ( mat33 & vrm,  realtype alpha, 
+extern void GetVecRotMatrix ( mat33 & vrm,  realtype alpha, 
                            realtype vx,  realtype vy, realtype vz );
 
 
@@ -854,11 +881,11 @@ extern void  GetVecRotMatrix ( mat33 & vrm,  realtype alpha,
 //  rotation axis vector (vx,vy,vz).
 //    The rotation angle and vector are determined up to
 //  their sign (however correlated, so that being substituted
-//  into GetVecRorMatrix(..) they yield the same rotation
+//  into GetVecRotMatrix(..) they yield the same rotation
 //  matrix).
 //    The function does not check for vrm to be a valid
 //  rotation matrix.
-extern void  GetRotParameters ( mat33 & vrm, realtype & alpha, 
+extern void GetRotParameters ( mat33 & vrm, realtype & alpha, 
                        realtype & vx, realtype & vy, realtype & vz );
 
 
@@ -867,9 +894,9 @@ extern void  GetRotParameters ( mat33 & vrm, realtype & alpha,
 // (vx,vy,vz) = (vx2-vx1,vy2-vy1,vz2-vz1). Point (x0,y0,z0) is
 // the center of rotation -- actually a point belonging to the
 // rotation axis.
-extern void  GetVecTMatrix  ( mat44 & vrm, realtype alpha, 
-                              realtype vx, realtype vy, realtype vz,
-                              realtype x0, realtype y0, realtype z0 );
+extern void GetVecTMatrix  ( mat44 & vrm, realtype alpha, 
+                             realtype vx, realtype vy, realtype vz,
+                             realtype x0, realtype y0, realtype z0 );
 
 //   Vector rotation is rotation by angle alpha about arbitrary
 // vector directed as (vx,vy,vz) = (vx2-vx1,vy2-vy1,vz2-vz1).
@@ -913,5 +940,11 @@ extern int SuperposeAtoms ( mat44 & T, PPCAtom A1, int nA, PPCAtom A2,
 extern void  SortContacts ( PSContact contact, int ncontacts,
                             int sortmode );
 
+
+#define  NO_TORSION  (-MaxReal)
+
+extern realtype getPhi ( PPCAtom A );  // A[0] - A[3] used
+extern realtype getPsi ( PPCAtom A );  // A[0] - A[2] used
+
 #endif
 
