@@ -59,6 +59,7 @@ typedef  ChainID *    PChainID;       // pointer to chain ID
 typedef  char         InsCode [10];   // insertion code
 typedef  char         DBName  [10];   // sequence database name
 typedef  char         DBAcCode[20];   // seq. database accession code
+typedef  DBAcCode *   PDBAcCode;      // pointer to seq. db acc code
 typedef  char         DBIdCode[20];   // seq. database ident-n code
 typedef  DBIdCode *   PDBIdCode;      // pointer to DBIdCode
 typedef  char         ResName [20];   // residue name
@@ -71,12 +72,15 @@ typedef  char         TurnID  [20];   // turn ID
 
 typedef  char         SymGroup[100];  // group of space symmetry
 typedef  realtype     vect3   [3];    // vector of 3 real numbers
+typedef  vect3    *   pvect3;
+typedef  pvect3   &   rpvect3;
 typedef  realtype     vect4   [4];    // vector of 4 real numbers
 typedef  vect3        mat33   [3];    // matrix 3x3 of real numbers
 
 typedef  vect4        mat44   [4];    // matrix 4x4 of real numbers
 typedef  mat44    *   pmat44;
 typedef  mat44    &   rmat44;
+typedef  pmat44   *   ppmat44;
 typedef  pmat44   &   rpmat44;
 typedef  mat33        mat633  [6];    // matrix 6x3x3 of real numbers
 
