@@ -22,7 +22,7 @@
 //
 //  =================================================================
 //
-//    08.07.08   <--  Date of Last Modification.
+//    24.03.09   <--  Date of Last Modification.
 //                   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //  -----------------------------------------------------------------
 //
@@ -33,7 +33,7 @@
 //  **** Classes :  CMMDBFile  ( macromolecular data file class )
 //       ~~~~~~~~~
 //
-//  (C) E. Krissinel 2000-2008
+//  (C) E. Krissinel 2000-2009
 //
 //  =================================================================
 //
@@ -102,6 +102,7 @@ void  CMMDBFile::InitMMDBFile()  {
   Exclude = True;
   ignoreRemarks = False;  // used temporarily
   allowDuplChID = False;  // used temporarily
+  modelCnt      = 0;      // used only at reading files
 }
 
 
@@ -140,13 +141,15 @@ void  CMMDBFile::FreeCoordMemory()  {
   AtmLen  = 0;
   nAtoms  = 0;
 
+  modelCnt = 0;
+
 }
 
 void  CMMDBFile::FreeFileMemory()  {
 
-  FreeCoordMemory ();
-  Title.FreeMemory();
-  Cryst.FreeMemory();
+  FreeCoordMemory  ();
+  Title.FreeMemory ( False );
+  Cryst.FreeMemory ();
 
   SA      .FreeContainer();
   Footnote.FreeContainer();
@@ -398,7 +401,7 @@ Boolean      fixSpaceGroup,fend;
   // read coordinate section
   RC = 0;
   do {
-    RC = ReadPDBAtom(S);
+    RC = ReadPDBAtom ( S );
     if (RC)  break;
     fend = f.FileEnd();
     if (!fend)  {
@@ -1597,9 +1600,10 @@ int RC,index;
 
   } else if (!strncmp(L,"MODEL ",6)) {
 
+    modelCnt++;
     RC = SwitchModel ( L );
     if (!RC)  {
-      if (crModel->serNum!=nModels)
+      if (crModel->serNum!=modelCnt)
         RC = Error_DuplicatedModel;
     }
 
@@ -1631,7 +1635,7 @@ int         RC,i,index,nATS;
 
   for (i=1;i<=nATS;i++)  {
     // nAtoms and i should always coincide at this point. This piece
-    // of code was however left in order to reach identicity with
+    // of code was however left in order to reach identity with
     // ReadPDBAtom(..).
     index = nAtoms+1;  // index for the next atom in Atom array
     RC    = CheckAtomPlace ( index,Loop );
