@@ -22,7 +22,7 @@
 //
 //  =================================================================
 //
-//    08.07.08   <--  Date of Last Modification.
+//    24.03.09   <--  Date of Last Modification.
 //                   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //  -----------------------------------------------------------------
 //
@@ -33,7 +33,7 @@
 //  **** Classes :  CMMDBFile
 //       ~~~~~~~~~
 //
-//  (C) E. Krissinel 2000-2008
+//  (C) E. Krissinel 2000-2009
 //
 //  =================================================================
 //
@@ -594,6 +594,9 @@ class CMMDBFile : public CUDData  {
 
     virtual PCMask GetSelMask ( int selHnd );
 
+  private :
+    int modelCnt;  // used only at reading files
+
 };
 
 
