@@ -22,7 +22,7 @@
 //
 //  =================================================================
 //
-//    14.05.07   <--  Date of Last Modification.
+//    17.03.09   <--  Date of Last Modification.
 //                   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //  -----------------------------------------------------------------
 //
@@ -33,7 +33,7 @@
 //  **** Classes :  CMMDBManager  ( MMDB file manager class )
 //       ~~~~~~~~~
 //
-//  (C) E. Krissinel 2000-2008
+//  (C) E. Krissinel 2000-2009
 //
 //  =================================================================
 //
@@ -184,8 +184,9 @@ int      i,j,nm, nchains;
 
   if (DelMask & MMDBFCM_Flags)  Flags = 0;
 
-  if (DelMask & MMDBFCM_Title)  Title.Copy ( NULL );
-  if (DelMask & MMDBFCM_Cryst)  Cryst.Copy ( NULL );
+  if (DelMask & MMDBFCM_Title)        Title.Copy ( NULL );
+  if (DelMask & MMDBFCM_TitleKeepBM)  Title.FreeMemory ( True );
+  if (DelMask & MMDBFCM_Cryst)        Cryst.Copy ( NULL );
 
   if (DelMask & MMDBFCM_Coord)  {
     FreeCoordMemory    ();
@@ -216,6 +217,14 @@ int      i,j,nm, nchains;
           model[i]->RemoveLinks();
   }
 
+  if (DelMask & MMDBFCM_CisPeps)  {
+    GetModelTable ( model,nm );
+    if (model)
+      for (i=0;i<nm;i++)
+        if (model[i])
+          model[i]->RemoveCisPeps();
+  }
+
   if (DelMask & MMDBFCM_ChainAnnot)  {
     nm = GetNumberOfModels();
     for (i=1;i<=nm;i++)  {
@@ -243,6 +252,74 @@ PCTitleContainer CMMDBManager::GetRemarks()  {
   return Title.GetRemarks();
 }
 
+realtype CMMDBManager::GetResolution()  {
+  return Title.GetResolution();
+}
+
+int CMMDBManager::ParseBiomolecules()  {
+  return Title.ParseBiomolecules();
+}
+
+int CMMDBManager::GetNofBiomolecules()  {
+  return Title.GetNofBiomolecules();
+}
+
+void CMMDBManager::GetBiomolecules ( PPCBiomolecule & BM,
+                                     int & nBMs )  {
+  Title.GetBiomolecules ( BM,nBMs );
+}
+
+PCBiomolecule CMMDBManager::GetBiomolecule ( int bmNo )  {
+  return Title.GetBiomolecule ( bmNo );
+}
+
+PCMMDBManager CMMDBManager::MakeBiomolecule ( int bmNo, int modelNo ) {
+PCMMDBManager M;
+PPCChain      ch;
+PCChain       chain;
+PCModel       model;
+PCBiomolecule BM;
+int           i,j,k,n,n0,nChains;
+
+  BM = Title.GetBiomolecule ( bmNo );
+  if (!BM)  return NULL;
+
+  GetChainTable ( modelNo,ch,nChains );
+  if ((!ch) || (nChains<=0))  return NULL;
+
+  n0    = 0;
+  model = new CModel();
+
+  for (i=0;(i<BM->nBMAs) && (n0>=0);i++)
+    if (BM->BMApply[i])  {
+      for (j=0;(j<BM->BMApply[i]->nMatrices) && (n0>=0);j++)
+        for (k=0;(k<BM->BMApply[i]->nChains) && (n0>=0);k++)  {
+          n0 = -1;
+          for (n=0;(n<nChains) && (n0<0);n++)
+            if (!strcmp(ch[n]->GetChainID(),BM->BMApply[i]->chain[k]))
+              n0 = n;
+          if (n0>=0)  {
+            chain = new CChain();
+            chain->Copy ( ch[n0] );
+            chain->ApplyTransform ( BM->BMApply[i]->tm[j] );
+            model->AddChain ( chain );
+          }
+        }
+    }
+
+  if (n0>=0)  {
+    M = new CMMDBManager();
+    M->AddModel ( model );
+    M->PDBCleanup ( PDBCLEAN_SERIAL | PDBCLEAN_INDEX );
+  } else  {
+    delete model;
+    M = NULL;
+  }
+
+  return M;
+
+}
+
 
 //  -------------------  Stream functions  ----------------------
 
