@@ -22,7 +22,7 @@
 //
 //  =================================================================
 //
-//    14.05.07   <--  Date of Last Modification.
+//    17.03.09   <--  Date of Last Modification.
 //                   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //  -----------------------------------------------------------------
 //
@@ -33,7 +33,7 @@
 //  **** Classes :  CMMDBManager  ( MMDB file manager )
 //       ~~~~~~~~~
 //
-//  (C) E. Krissinel 2000-2008
+//  (C) E. Krissinel 2000-2009
 //
 //  =================================================================
 //
@@ -52,19 +52,21 @@
 // copy masks
 #define MMDBFCM_All            0xFFFFFFFF
 #define MMDBFCM_Title          0x00000001
-#define MMDBFCM_Cryst          0x00000002
-#define MMDBFCM_Coord          0x00000004
-#define MMDBFCM_SecStruct      0x00000008
-#define MMDBFCM_HetInfo        0x00000010
-#define MMDBFCM_Links          0x00000020
-#define MMDBFCM_SA             0x00000040
-#define MMDBFCM_SB             0x00000080
-#define MMDBFCM_SC             0x00000100
-#define MMDBFCM_Footnotes      0x00000200
-#define MMDBFCM_ChainAnnot     0x00000400
-#define MMDBFCM_Flags          0x00000800
+#define MMDBFCM_TitleKeepBM    0x00000002
+#define MMDBFCM_Cryst          0x00000004
+#define MMDBFCM_Coord          0x00000008
+#define MMDBFCM_SecStruct      0x00000010
+#define MMDBFCM_HetInfo        0x00000020
+#define MMDBFCM_Links          0x00000040
+#define MMDBFCM_CisPeps        0x00000080
+#define MMDBFCM_SA             0x00000100
+#define MMDBFCM_SB             0x00000200
+#define MMDBFCM_SC             0x00000400
+#define MMDBFCM_Footnotes      0x00000800
+#define MMDBFCM_ChainAnnot     0x00001000
+#define MMDBFCM_Flags          0x00002000
 #define MMDBFCM_Buffer         0x80000000
-#define MMDBFCM_Top            0xFFFFFFFB
+#define MMDBFCM_Top            0xFFFFFFF7
 
 DefineStreamFunctions(CMMDBManager)
 
@@ -93,6 +95,18 @@ class CMMDBManager : public CMMDBBondManager  {
 
     PCTitleContainer GetRemarks();
 
+    realtype GetResolution(); // -1.0 means no resolution record in file
+
+    int   ParseBiomolecules(); // returns the number of biomolecules,
+                               // -2 for general format error
+                               // -3 for errors in BIOMT records
+    int   GetNofBiomolecules();
+    void  GetBiomolecules   ( PPCBiomolecule & BM, int & nBMs );
+
+    PCBiomolecule GetBiomolecule ( int bmNo ); // bmno=0,1,..
+                               // returns NULL if bmNo is incorrect
+    PCMMDBManager MakeBiomolecule ( int bmNo, int modelNo=1 );
+
   protected :
 
     //  ---------------  Stream I/O  -----------------------------
