@@ -1220,7 +1220,7 @@ int j,k,r1,r2;
       S[j] = NULL;
       if (field[j])  {
         if (field[j][k])  {
-	  if (field[j][k][0]!=char(2))
+          if (field[j][k][0]!=char(2))
             CreateCopy ( S[j],field[j][k] );
         }
       }
@@ -1613,7 +1613,7 @@ CMMCIFData::CMMCIFData ( RPCStream Object ) : CStream(Object)  {
 }
 
 CMMCIFData::~CMMCIFData() {
-  FreeMemory();
+  FreeMemory(0);
 }
 
 void CMMCIFData::InitMMCIFData()  {
@@ -1912,8 +1912,8 @@ int           RC,i;
     } else  {
       Warning |= CIFW_DuplicatedTag;
       if (flags & CIFFL_PrintWarnings)
-      printf ( "\n **** mmCIF READ WARNING "
-               "<<line %i: duplicated tag>>\n%s\n",lcount,S );
+        printf ( "\n **** mmCIF READ WARNING "
+                 "<<line %i: duplicated tag>>\n%s\n",lcount,S );
     }
   }
   Struct->AddField ( L,T );
@@ -1956,71 +1956,93 @@ int         RC,i,nC;
         T[0] = char(1);  // special category name
       }
 
-      //  look for category
-      i = AddCategory ( T );
-      if ((i!=nC) && (nC>=0))  {  // empty loop; most probably
-                                  // a corrupted file
-        p = p1;   // play back to last category
-        Warning |= CIFW_EmptyLoop;
-        if (flags & CIFFL_PrintWarnings)
-          printf ( "\n **** mmCIF READ WARNING "
-                   "<<line %i: empty loop>>\n%s\n",lcount,S );
-        // AddCategory(..) has added a NULL-Category on the top of
-        // category list; remove it now
-        DeleteCategory ( nCategories-1 );
-        return;
-      }
-      if (i<0)  {
-        // negative value means that the category was not in the list,
-        // but a place for it has been provided and index updated
-        Loop = new CMMCIFLoop ( T );
-        Category[nCategories-1] = Loop;
-        nC = nCategories-1;
+      if (Loop)  {
+        if (strcmp(Loop->GetCategoryName(),T))  {
+          // loop ended, empty
+          p    = p1;   // play back to last category
+          Loop = NULL;
+        }
       } else  {
-        Loop = PCMMCIFLoop(Category[i]);
-        if (Loop->GetCategoryID()!=MMCIF_Loop)  {
-          Warning |= CIFW_NotALoop;
+        //  look for category
+        i = AddCategory ( T );
+        if ((i!=nC) && (nC>=0))  {  // empty loop; most probably
+                                    // a corrupted file
+          p = p1;   // play back to last category
+          Warning |= CIFW_EmptyLoop;
           if (flags & CIFFL_PrintWarnings)
             printf ( "\n **** mmCIF READ WARNING "
-                     "<<line %i: %s was a structure --"
-                     " replaced>>\n%s\n",lcount,T,S );
-          delete Category[i];
+                     "<<line %i: empty loop>>\n%s\n",lcount,S );
+          // AddCategory(..) has added a NULL-Category on the top of
+          // category list; remove it now
+          DeleteCategory ( nCategories-1 );
+          Loop = NULL;
+//          return;
+        }
+        if (i<0)  {
+          // negative value means that the category was not in the list,
+          // but a place for it has been provided and index updated
           Loop = new CMMCIFLoop ( T );
-          Category[i] = Loop;
+          Category[nCategories-1] = Loop;
+          nC = nCategories-1;
         }
       }
-  
-      if (*p=='.')  {  // get item name
-        i = 0;
-        p++;  // skip period
-        while ((*p) && (*p!=' ') && (*p!=char(9)))  {
-          T[i++] = *p;
-          p++;
-        }
-        T[i] = char(0);
-      } else
-        strcpy ( T,L );
-
-
-      if (nWrongFields>0)
-            WrongField = CheckWrongField ( Loop->name,T );
-      else  WrongField = False;
-
-      if (!WrongField)  {
-        if (Loop->AddTag(T)>=0)  {
-          if (flags & CIFFL_SuggestTags)  {
-            tagNo++;
-            ParamStr ( T,pstr("\1"),tagNo );
-            Loop->AddTag(T);
-          } else  {
-            Warning |= CIFW_DuplicatedTag;
+/*
+ else if (Loop)  {
+        if (!strcmp(Loop->GetCategoryName(),
+                    Category[i]->GetCategoryName()))  {
+          if (Loop->GetCategoryID()!=MMCIF_Loop)  {
+            Warning |= CIFW_NotALoop;
             if (flags & CIFFL_PrintWarnings)
               printf ( "\n **** mmCIF READ WARNING "
-                       "<<line %i: duplicated tag>>\n%s\n",lcount,S );
+                       "<<line %i: %s was a structure --"
+                       " replaced>>\n%s\n",lcount,T,S );
+            delete Category[i];
+            Loop = new CMMCIFLoop ( T );
+            Category[i] = Loop;
+          }
+        } else
+          Loop = NULL;
+      }
+*/
+      if (Loop)  {
+  
+        if (*p=='.')  {  // get item name
+          i = 0;
+          p++;  // skip period
+          while ((*p) && (*p!=' ') && (*p!=char(9)))  {
+            T[i++] = *p;
+            p++;
+          }
+          T[i] = char(0);
+        } else
+          strcpy ( T,L );
+
+        if (nWrongFields>0)
+              WrongField = CheckWrongField ( Loop->name,T );
+        else  WrongField = False;
+ 
+        if (!WrongField)  {
+          if (Loop->AddTag(T)>=0)  {
+            if (flags & CIFFL_SuggestTags)  {
+              tagNo++;
+              ParamStr ( T,pstr("\1"),tagNo );
+              Loop->AddTag(T);
+            } else  {
+              Warning |= CIFW_DuplicatedTag;
+              if (flags & CIFFL_PrintWarnings)
+                printf ( "\n **** mmCIF READ WARNING "
+                         "<<line %i: duplicated tag>>\n%s\n",lcount,S );
+            }
           }
         }
+        Repeat = True;
+
+      } else  {
+
+        p = p1;
+        Repeat = False;
+
       }
-      Repeat = True;
 
     } else if (!(*p) || (*p=='#'))  {
       Repeat = !f.FileEnd();
@@ -3059,7 +3081,7 @@ int i,RC;
 
 void CMMCIFData::Copy ( PCMMCIFData Data )  {
 int i;
-  FreeMemory();
+  FreeMemory(0);
   CreateCopy ( name,Data->name );
   nCategories = Data->nCategories;
   if (nCategories>0)  {
@@ -3162,7 +3184,7 @@ int i,k;
 
 void CMMCIFData::read ( RCFile f )  {
 int i,k;
-  FreeMemory();
+  FreeMemory(0);
   f.CreateRead ( name );
   f.ReadInt    ( &nCategories );
   if (nCategories>0)  {
@@ -3248,9 +3270,9 @@ int         RC;
   CIF = NULL;
   f.assign ( FName,True,False,gzipMode );
   if (f.reset(True))  {
-    S[0]   = char(0);
-    lcount = 0;
-    RC     = 0;
+    S[0]       = char(0);
+    lcount     = 0;
+    RC         = 0;
     while ((!RC) && (!f.FileEnd()))  {
       if (!CIF)  CIF = new CMMCIFData();
       CIF->SetPrintWarnings ( PrintWarnings );
@@ -3272,7 +3294,7 @@ int         RC;
     return RC;
   } else
     return CIFRC_CantOpenFile;
-  
+
 }
 
 int CMMCIFFile::WriteMMCIFFile ( cpstr FName, byte gzipMode )  {
