@@ -503,7 +503,7 @@ class CMMCIFData : public CStream  {
     //   Unless the data are to be added to the existing CIF structure,
     // FreeMemory() should be called once before creating a new
     // CIF data set.
-    void FreeMemory ( int key=0 );
+    void FreeMemory ( int key );
 
     void PutDataName ( cpstr dname ); // stores name for 'data_'
                                            // record
@@ -664,7 +664,7 @@ class CMMCIFFile : public CStream  {
     int   ReadMMCIFFile    ( cpstr FName,byte gzipMode=GZM_CHECK);
     int   WriteMMCIFFile   ( cpstr FName,byte gzipMode=GZM_CHECK);
 
-    int   GetNofData() { return nData; }
+    int   GetNofData()  { return nData; }
     PCMMCIFData GetCIFData ( int        dataNo );  // 0..nData-1
     PCMMCIFData GetCIFData ( cpstr DName  );
     int   AddMMCIFData     ( cpstr DName  );
