@@ -1,53 +1,28 @@
-//  $Id: mmdb_model.h,v 1.7 2004/06/08 16:26:18 keb Exp $
+//  $Id: mmdb_model.h,v 1.12 2008/07/08 15:28:16 keb Exp $
 //  =================================================================
 //
 //   CCP4 Coordinate Library: support of coordinate-related
 //   functionality in protein crystallography applications.
 //
-//   Copyright (C) Eugene Krissinel 2004.
+//   Copyright (C) Eugene Krissinel 2000-2008.
 //
-//   This library is free software and is distributed under the terms
-//   and conditions of the CCP4 licence agreement as `Part 0' (Annex 2)
-//   software, which is version 2.1 of the GNU Lesser General Public
-//   Licence (LGPL) with the following additional clause:
+//    This library is free software: you can redistribute it and/or 
+//    modify it under the terms of the GNU Lesser General Public 
+//    License version 3, modified in accordance with the provisions 
+//    of the license to address the requirements of UK law.
 //
-//      `You may also combine or link a "work that uses the Library"
-//      to produce a work containing portions of the Library, and
-//      distribute that work under terms of your choice, provided that
-//      you give prominent notice with each copy of the work that the
-//      specified version of the Library is used in it, and that you
-//      include or provide public access to the complete corresponding
-//      machine-readable source code for the Library including whatever
-//      changes were used in the work. (i.e. If you make changes to the
-//      Library you must distribute those, but you do not need to
-//      distribute source or object code to those portions of the work
-//      not covered by this licence.)'
+//    You should have received a copy of the modified GNU Lesser 
+//    General Public License along with this library. If not, copies 
+//    may be downloaded from http://www.ccp4.ac.uk/ccp4license.php
 //
-//   Note that this clause grants an additional right and does not
-//   impose any additional restriction, and so does not affect
-//   compatibility with the GNU General Public Licence (GPL). If you
-//   wish to negotiate other terms, please contact the maintainer.
-//
-//   You can redistribute it and/or modify the library under the terms
-//   of the GNU Lesser General Public License as published by the Free
-//   Software Foundation; either version 2.1 of the License, or (at
-//   your option) any later version.
-//
-//   This library is distributed in the hope that it will be useful,
-//   but WITHOUT ANY WARRANTY; without even the implied warranty of
-//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-//   Lesser General Public License for more details.
-//
-//   You should have received a copy of the CCP4 licence and/or GNU
-//   Lesser General Public License along with this library; if not,
-//   write to the CCP4 Secretary, Daresbury Laboratory, Warrington
-//   WA4 4AD, UK. The GNU Lesser General Public can also be obtained
-//   by writing to the Free Software Foundation, Inc., 59 Temple Place,
-//   Suite 330, Boston, MA 02111-1307 USA
+//    This program is distributed in the hope that it will be useful,
+//    but WITHOUT ANY WARRANTY; without even the implied warranty of
+//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+//    GNU Lesser General Public License for more details.
 //
 //  =================================================================
 //
-//    28.05.04   <--  Date of Last Modification.
+//    08.07.08   <--  Date of Last Modification.
 //                   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //  -----------------------------------------------------------------
 //
@@ -63,9 +38,13 @@
 //                  CSheet        ( sheet info                      )
 //                  CSheets       ( container for sheets            )
 //                  CTurn         ( turn info                       )
+//                  CLinkContainer   ( container for link data      )
+//                  CLink            ( link data                    )
+//                  CCisPepContainer ( container for CisPep data    )
+//                  CCisPep          ( CisPep data                  )
 //                  CModel        ( PDB model                       )
 //
-//  Copyright (C) E. Krissinel 2000-2004
+//  Copyright (C) E. Krissinel 2000-2008
 //
 //  =================================================================
 //
@@ -110,11 +89,11 @@ class CHetCompound : public CStream  {
     char     wc;         // '*' for water, otherwise space
     pstr     Formula;    // formulas
 
-    CHetCompound ( const pstr HetName );
+    CHetCompound ( cpstr HetName );
     CHetCompound ( RPCStream Object );
     ~CHetCompound();
 
-    void  AddKeyWord     ( const pstr W, Boolean Closed );
+    void  AddKeyWord     ( cpstr W, Boolean Closed );
     void  HETNAM_PDBDump ( RCFile f );
     void  HETSYN_PDBDump ( RCFile f );
     void  FORMUL_PDBDump ( RCFile f );
@@ -130,7 +109,7 @@ class CHetCompound : public CStream  {
 
   protected :
 
-    void  InitHetCompound ( const pstr HetName );
+    void  InitHetCompound ( cpstr HetName );
     void  FreeMemory      ();
 
 };
@@ -178,13 +157,13 @@ class CHelix : public CContainerClass  {
     int     length;      // length of the helix
 
     CHelix ();
-    CHelix ( const pstr S );
+    CHelix ( cpstr S );
     CHelix ( RPCStream Object );
     ~CHelix();
 
     void  PDBASCIIDump    ( pstr S, int N   );
     void  MakeCIF         ( PCMMCIFData CIF, int N );
-    int   ConvertPDBASCII ( const pstr S );
+    int   ConvertPDBASCII ( cpstr S );
     void  GetCIF          ( PCMMCIFData CIF, int & Signal );
     int   GetClassID      () { return ClassID_Helix; }
 
@@ -220,16 +199,21 @@ class CStrand : public CStream  {
     ChainID  endChainID;  // chain ID of terminal residue in the strand
     int      endSeqNum;   // sequence number of the terminal residue
     InsCode  endICode;    // insertion code of the terminal residue
-    int      sense;       // sense of strand with respect to previous strand
+    int      sense;       // sense of strand with respect to previous
+                          //    strand
     AtomName curAtom;     // registration; atom name in current strand
-    ResName  curResName;  // registration; residue name in current strand
+    ResName  curResName;  // registration; residue name in current
+                          //    strand
     ChainID  curChainID;  // registration; chain ID in current strand
-    int      curResSeq;   // registration; res-e seq numb in current strand
+    int      curResSeq;   // registration; res-e seq numb in current
+                          //    strand
     InsCode  curICode;    // registration; ins code in current strand
     AtomName prevAtom;    // registration; atom name in previous strand
-    ResName  prevResName; // registration; residue name in previous strand
+    ResName  prevResName; // registration; residue name in previous
+                          //    strand
     ChainID  prevChainID; // registration; chain ID in previous strand
-    int      prevResSeq;  // registration; res-e seq numb in previous strand
+    int      prevResSeq;  // registration; res-e seq numb in previous
+                          //    strand
     InsCode  prevICode;   // registration; ins code in previous strand
 
     CStrand ();
@@ -238,8 +222,8 @@ class CStrand : public CStream  {
 
     void  PDBASCIIDump    ( pstr S          );
     void  MakeCIF         ( PCMMCIFData CIF );
-    int   ConvertPDBASCII ( const pstr S    );
-    int   GetCIF          ( PCMMCIFData CIF, const pstr sheet_id );
+    int   ConvertPDBASCII ( cpstr S    );
+    int   GetCIF          ( PCMMCIFData CIF, cpstr sheet_id );
 
     void  Copy  ( PCStrand Strand );
 
@@ -274,7 +258,7 @@ class CSheet : public CStream  {
 
     void  PDBASCIIDump    ( RCFile f        );
     void  MakeCIF         ( PCMMCIFData CIF );
-    int   ConvertPDBASCII ( const pstr  S   );
+    int   ConvertPDBASCII ( cpstr  S   );
     int   GetCIF          ( PCMMCIFData CIF );
 
     void  Copy  ( PCSheet Sheet );
@@ -284,7 +268,7 @@ class CSheet : public CStream  {
 
   protected :
     void  InitSheet      ();
-    void  CIFFindStrands ( PCMMCIFData CIF, const pstr Category );
+    void  CIFFindStrands ( PCMMCIFData CIF, cpstr Category );
     void  TryStrand      ( int strand_no );
     int   GetStrand      ( int strand_no );
 
@@ -310,7 +294,7 @@ class CSheets : public CStream  {
 
     void  PDBASCIIDump    ( RCFile f );
     void  MakeCIF         ( PCMMCIFData CIF );
-    int   ConvertPDBASCII ( const pstr  S   );
+    int   ConvertPDBASCII ( cpstr  S   );
     int   GetCIF          ( PCMMCIFData CIF );
 
     void  Copy  ( PCSheets Sheets );
@@ -320,7 +304,7 @@ class CSheets : public CStream  {
 
   protected :
     void  InitSheets    ();
-    void  CIFFindSheets ( PCMMCIFData CIF, const pstr Category );
+    void  CIFFindSheets ( PCMMCIFData CIF, cpstr Category );
 
 };
 
@@ -346,13 +330,13 @@ class CTurn : public CContainerClass  {
     pstr    comment;     // comment about the helix
 
     CTurn ();
-    CTurn ( const pstr S );
+    CTurn ( cpstr S );
     CTurn ( RPCStream Object );
     ~CTurn();
 
     void  PDBASCIIDump    ( pstr S, int N   );
     void  MakeCIF         ( PCMMCIFData CIF, int N );
-    int   ConvertPDBASCII ( const pstr S );
+    int   ConvertPDBASCII ( cpstr S );
     void  GetCIF          ( PCMMCIFData CIF, int & Signal );
     int   GetClassID      () { return ClassID_Turn; }
 
@@ -388,9 +372,9 @@ class CHetCompounds : public CStream  {
     void  FreeMemory    ();
 
     void  PDBASCIIDump  ( RCFile f );
-    void  ConvertHETNAM ( const pstr S );
-    void  ConvertHETSYN ( const pstr S );
-    void  ConvertFORMUL ( const pstr S );
+    void  ConvertHETNAM ( cpstr S );
+    void  ConvertHETSYN ( cpstr S );
+    void  ConvertFORMUL ( cpstr S );
 
     void  MakeCIF       ( PCMMCIFData CIF );
     void  GetCIF        ( PCMMCIFData CIF );
@@ -404,7 +388,137 @@ class CHetCompounds : public CStream  {
     Boolean Closed;
 
     void  InitHetCompounds();
-    int   AddHetName      ( const pstr H );
+    int   AddHetName      ( cpstr H );
+
+};
+
+
+//  ===================  CLinkContainer  =====================
+
+DefineClass(CLinkContainer)
+DefineStreamFunctions(CLinkContainer)
+
+class CLinkContainer : public CClassContainer  {
+
+  public :
+
+    CLinkContainer  () : CClassContainer() {}
+    CLinkContainer  ( RPCStream Object )
+                     : CClassContainer ( Object ) {} 
+    ~CLinkContainer () {}
+
+    PCContainerClass MakeContainerClass ( int ClassID );
+
+};
+
+
+//  ====================  CLink  ============================
+
+DefineClass(CLink)
+DefineStreamFunctions(CLink)
+
+class CLink : public CContainerClass  {
+
+  public :
+    AtomName atName1;   // name of 1st linked atom
+    AltLoc   aloc1;     // alternative location of 1st linked atom
+    ResName  resName1;  // residue name of 1st linked atom
+    ChainID  chainID1;  // chain ID of 1st linked atom
+    int      seqNum1;   // sequence number of 1st linked atom
+    InsCode  insCode1;  // insertion code of 1st linked atom
+    AtomName atName2;   // name of 2nd linked atom
+    AltLoc   aloc2;     // alternative location of 2nd linked atom
+    ResName  resName2;  // residue name of 2nd linked atom
+    ChainID  chainID2;  // chain ID of 2nd linked atom
+    int      seqNum2;   // sequence number of 2nd linked atom
+    InsCode  insCode2;  // insertion code of 2nd linked atom
+    int      s1,i1,j1,k1;  // sym id of 1st atom
+    int      s2,i2,j2,k2;  // sym id of 2nd atom
+    
+    CLink ();
+    CLink ( cpstr S );
+    CLink ( RPCStream Object );
+    ~CLink();
+
+    void  PDBASCIIDump    ( pstr S, int N   );
+    void  MakeCIF         ( PCMMCIFData CIF, int N );
+    int   ConvertPDBASCII ( cpstr S );
+    void  GetCIF          ( PCMMCIFData CIF, int & Signal );
+    int   GetClassID      () { return ClassID_Link; }
+
+    void  Copy  ( PCContainerClass Link );
+
+    void  write ( RCFile f );
+    void  read  ( RCFile f );
+
+  protected :
+    
+    void InitLink();
+
+};
+
+
+
+//  ===================  CCisPepContainer  =====================
+
+DefineClass(CCisPepContainer)
+DefineStreamFunctions(CCisPepContainer)
+
+class CCisPepContainer : public CClassContainer  {
+
+  public :
+
+    CCisPepContainer  () : CClassContainer() {}
+    CCisPepContainer  ( RPCStream Object )
+                     : CClassContainer ( Object ) {} 
+    ~CCisPepContainer () {}
+
+    PCContainerClass MakeContainerClass ( int ClassID );
+
+};
+
+
+//  =====================  CCisPep  ===========================
+
+DefineClass(CCisPep)
+DefineStreamFunctions(CCisPep)
+
+class CCisPep : public CContainerClass  {
+
+  public :
+    int      serNum;   //  record serial number
+    ResName  pep1;     //  residue name
+    ChainID  chainID1; //  chain identifier 1
+    int      seqNum1;  //  residue sequence number 1
+    InsCode  icode1;   //  insertion code 1
+    ResName  pep2;     //  residue name 2
+    ChainID  chainID2; //  chain identifier 2
+    int      seqNum2;  //  residue sequence number 2
+    InsCode  icode2;   //  insertion code 2
+    int      modNum;   //  model number
+    realtype measure;  //  measure of the angle in degrees.
+
+    CCisPep ();
+    CCisPep ( cpstr S );
+    CCisPep ( RPCStream Object );
+    ~CCisPep();
+
+    void  PDBASCIIDump    ( pstr S, int N );
+    int   ConvertPDBASCII ( cpstr S );
+
+//    void  MakeCIF         ( PCMMCIFData CIF, int N );
+//    void  GetCIF          ( PCMMCIFData CIF, int & Signal );
+
+    int   GetClassID      () { return ClassID_CisPep; }
+
+    void  Copy  ( PCContainerClass CisPep );
+
+    void  write ( RCFile f );
+    void  read  ( RCFile f );
+
+  protected :
+
+    void InitCisPep();
 
 };
 
@@ -460,9 +574,10 @@ class CModel : public CProModel  {
     pstr   GetModelID ( pstr modelID );  // returns "/mdl"
 
     int    GetNumberOfModels  (); // returns TOTAL number of models
-    int    GetNumberOfAtoms   ( Boolean countTers );
-                                  // returns number of atoms in the model
-    int    GetNumberOfResidues(); // returns number of residues in the model
+    int    GetNumberOfAtoms   ( Boolean countTers ); // returns number
+                                              // of atoms in the model
+    int    GetNumberOfResidues(); // returns number of residues in
+                                  // the model
 
 
     //  ----------------  Extracting chains  --------------------------
@@ -507,7 +622,8 @@ class CModel : public CProModel  {
                            const InsCode insCode );
     void GetResidueTable ( PPCResidue & resTable,
                            int & NumberOfResidues );
-    void GetResidueTable ( const ChainID chainID, PPCResidue & resTable,
+    void GetResidueTable ( const ChainID chainID,
+                           PPCResidue & resTable,
                            int & NumberOfResidues );
     void GetResidueTable ( int   chainNo, PPCResidue & resTable,
                            int & NumberOfResidues );
@@ -522,10 +638,10 @@ class CModel : public CProModel  {
     int DeleteResidue ( int   chainNo, int resNo );
     int DeleteAllResidues ( const ChainID chainID );
     int DeleteAllResidues ( int   chainNo );
-    int DeleteSolvent     ();  // in difference of DeleteSolventChains,
-                               // this will remove all solvent molecules
-                               // from the file rather then solely-solvent
-                               // chains
+    int DeleteSolvent     (); // in difference of DeleteSolventChains,
+                              // this will remove all solvent molecules
+                              // from the file rather then
+                              // solely-solvent chains
     int DeleteAllResidues ();
 
     //  ------------------  Adding residues  --------------------------
@@ -535,7 +651,8 @@ class CModel : public CProModel  {
 
     //  -------------------  Extracting atoms  ------------------------
 
-    int GetNumberOfAllAtoms(); // returns TOTAL number of atoms in all models
+    int GetNumberOfAllAtoms(); // returns TOTAL number of atoms in all
+                               //    models
     PPCAtom    GetAllAtoms (); // returns pointer to Atom array
 
     int   GetNumberOfAtoms ( const ChainID chainID, int seqNo,
@@ -650,8 +767,8 @@ class CModel : public CProModel  {
     // occupancy, if those are equal or unspecified, the one with
     // "least" alternative location indicator.
     //  The function returns the number of deleted. All tables remain
-    // untrimmed, so that explicit trimming or calling FinishStructEdit()
-    // is required.
+    // untrimmed, so that explicit trimming or calling
+    // FinishStructEdit() is required.
     int DeleteAltLocs();
 
 
@@ -675,8 +792,14 @@ class CModel : public CProModel  {
     int   ConvertPDBString ( pstr PDBString );
 
     // PDBASCIIDumpPS(..) makes output of PDB primary structure records
+    // excluding cispeps
     void  PDBASCIIDumpPS   ( RCFile f );
-    // PDBASCIIDumpPS(..) makes output of PDB coordinate (ATOM etc.) records
+
+    // PDBASCIIDumpCP(..) makes output of cispep records
+    void  PDBASCIIDumpCP   ( RCFile f );
+
+    // PDBASCIIDump(..) makes output of PDB coordinate (ATOM etc.)
+    // records
     void  PDBASCIIDump     ( RCFile f );
 
     void  MakeAtomCIF      ( PCMMCIFData CIF );
@@ -723,37 +846,63 @@ class CModel : public CProModel  {
     int  GetNumberOfHelices ();
     int  GetNumberOfSheets  ();
 
-    PCHelix   GetHelix      ( int serialNum );   // 1<=serNum<=NofHelices
+    PCHelix   GetHelix      ( int serialNum ); // 1<=serNum<=NofHelices
 
     void      GetSheetID    ( int serialNum, SheetID sheetID );
                                                  // '\0' for none
 
-    PCSheet   GetSheet      ( int   serialNum ); // 1<=serNum<=NofSheets
+    PCSheet   GetSheet      ( int   serialNum ); //1<=serNum<=NofSheets
     PCSheet   GetSheet      ( const SheetID sheetID ); // NULL for none
     int  GetNumberOfStrands ( int   sheetSerNum );
     int  GetNumberOfStrands ( const SheetID sheetID );
-    PCStrand  GetStrand     ( int   sheetSerNum,     int strandSerNum );
-    PCStrand  GetStrand     ( const SheetID sheetID, int strandSerNum );
+    PCStrand  GetStrand     ( int   sheetSerNum,
+                              int strandSerNum );
+    PCStrand  GetStrand     ( const SheetID sheetID,
+                              int strandSerNum );
 
-    PCSSContainer GetHelices() { return &Helix;  }
-    PCSheets      GetSheets () { return &Sheets; }
+    PCSSContainer   GetHelices() { return &Helices;  }
+    PCSheets        GetSheets () { return &Sheets;   }
 
     void  RemoveSecStructure();
-    int   CalcSecStructure  ( Boolean flagBulge=True );
+    int   CalcSecStructure  ( Boolean flagBulge=True,
+                              int aminoSelHnd=-1 );
+//    int   CalcSecStructure  ( Boolean flagBulge=True );
 
     void  RemoveHetInfo     ();
 
+
+    //  ----  Working Links
+
+    int    GetNumberOfLinks ();
+    PCLink          GetLink ( int serialNum ); // 1<=serNum<=NofLinks
+    PCLinkContainer GetLinks() { return &Links; }
+
+    void   RemoveLinks();
+    void   AddLink    ( PCLink Link );
+
+
+    //  ----  Working CisPeps
+
+    int       GetNumberOfCisPeps();
+    PCCisPep          GetCisPep ( int CisPepNum );
+    PCCisPepContainer GetCisPeps() { return &CisPeps; }
+
+    void  RemoveCisPeps();
+    void  AddCisPep    ( PCCisPep CisPep );
+
+
+
     void  ApplyTransform    ( mat44 & TMatrix );  // transforms all
-                                                  // coordinates by multiplying
-                                                  // with matrix TMatrix
+                                      // coordinates by multiplying
+                                      // with matrix TMatrix
 
     Boolean isInSelection ( int selHnd );
 
 
     // -------  user-defined data handlers
-    int   PutUDData ( int UDDhandle, int        iudd );
-    int   PutUDData ( int UDDhandle, realtype   rudd );
-    int   PutUDData ( int UDDhandle, const pstr sudd );
+    int   PutUDData ( int UDDhandle, int      iudd );
+    int   PutUDData ( int UDDhandle, realtype rudd );
+    int   PutUDData ( int UDDhandle, cpstr    sudd );
 
     int   GetUDData ( int UDDhandle, int      & iudd );
     int   GetUDData ( int UDDhandle, realtype & rudd );
@@ -768,19 +917,21 @@ class CModel : public CProModel  {
 
   protected :
 
-    int             serNum;       // the model serial number
-    PCMMDBManager   manager;      // pointer to mmdbmanager class
+    int              serNum;       // the model serial number
+    PCMMDBManager    manager;      // pointer to mmdbmanager class
 
-    CHetCompounds   HetCompounds; // information on heterocompounds
-    CSSContainer    Helix;        // information on helices
-    CSheets         Sheets;       // information on sheets
-    CSSContainer    Turn;         // information on turns
+    CHetCompounds    HetCompounds; // information on heterocompounds
+    CSSContainer     Helices;      // information on helices
+    CSheets          Sheets;       // information on sheets
+    CSSContainer     Turns;        // information on turns
+    CLinkContainer   Links;        // information on links
+    CCisPepContainer CisPeps;      // information on cispeps
 
-    int             nChains;      // number of chains
-    int             nChainsAlloc; // actual length of Chain[]
-    PPCChain        Chain;        // array of chains
+    int              nChains;      // number of chains
+    int              nChainsAlloc; // actual length of Chain[]
+    PPCChain         Chain;        // array of chains
 
-    Boolean         Exclude;      // used internally
+    Boolean          Exclude;      // used internally
 
     void  InitModel        ();
     void  FreeMemory       ();
@@ -788,10 +939,12 @@ class CModel : public CProModel  {
     int   GetCIFPSClass    ( PCMMCIFData CIF, int ClassID );
 
     //   _ExcludeChain(..) excludes (but does not dispose!) a chain
-    // from the model. Returns 1 if the chain gets empty and 0 otherwise.
+    // from the model. Returns 1 if the chain gets empty and 0
+    // otherwise.
     int   _ExcludeChain ( const ChainID chainID );
 
-    //  _copy(PCModel) does not copy atoms! -- not for use in application
+    //  _copy(PCModel) does not copy atoms! -- not for use in
+    // applications
     void  _copy ( PCModel Model );
 
     //  _copy(PCModel,PPCAtom,int&) does copy atoms into array 'atom'
