@@ -1,53 +1,28 @@
-//  $Id: mmdb_sbase0.cpp,v 1.3 2005/09/22 16:32:39 keb Exp $
+//  $Id: mmdb_sbase0.cpp,v 1.8 2008/07/08 15:28:16 keb Exp $
 //  =================================================================
 //
 //   CCP4 Coordinate Library: support of coordinate-related
 //   functionality in protein crystallography applications.
 //
-//   Copyright (C) Eugene Krissinel 2004.
+//   Copyright (C) Eugene Krissinel 2000-2008.
 //
-//   This library is free software and is distributed under the terms
-//   and conditions of the CCP4 licence agreement as `Part 0' (Annex 2)
-//   software, which is version 2.1 of the GNU Lesser General Public
-//   Licence (LGPL) with the following additional clause:
+//    This library is free software: you can redistribute it and/or 
+//    modify it under the terms of the GNU Lesser General Public 
+//    License version 3, modified in accordance with the provisions 
+//    of the license to address the requirements of UK law.
 //
-//      `You may also combine or link a "work that uses the Library"
-//      to produce a work containing portions of the Library, and
-//      distribute that work under terms of your choice, provided that
-//      you give prominent notice with each copy of the work that the
-//      specified version of the Library is used in it, and that you
-//      include or provide public access to the complete corresponding
-//      machine-readable source code for the Library including whatever
-//      changes were used in the work. (i.e. If you make changes to the
-//      Library you must distribute those, but you do not need to
-//      distribute source or object code to those portions of the work
-//      not covered by this licence.)'
+//    You should have received a copy of the modified GNU Lesser 
+//    General Public License along with this library. If not, copies 
+//    may be downloaded from http://www.ccp4.ac.uk/ccp4license.php
 //
-//   Note that this clause grants an additional right and does not
-//   impose any additional restriction, and so does not affect
-//   compatibility with the GNU General Public Licence (GPL). If you
-//   wish to negotiate other terms, please contact the maintainer.
-//
-//   You can redistribute it and/or modify the library under the terms
-//   of the GNU Lesser General Public License as published by the Free
-//   Software Foundation; either version 2.1 of the License, or (at
-//   your option) any later version.
-//
-//   This library is distributed in the hope that it will be useful,
-//   but WITHOUT ANY WARRANTY; without even the implied warranty of
-//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-//   Lesser General Public License for more details.
-//
-//   You should have received a copy of the CCP4 licence and/or GNU
-//   Lesser General Public License along with this library; if not,
-//   write to the CCP4 Secretary, Daresbury Laboratory, Warrington
-//   WA4 4AD, UK. The GNU Lesser General Public can also be obtained
-//   by writing to the Free Software Foundation, Inc., 59 Temple Place,
-//   Suite 330, Boston, MA 02111-1307 USA
+//    This program is distributed in the hope that it will be useful,
+//    but WITHOUT ANY WARRANTY; without even the implied warranty of
+//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+//    GNU Lesser General Public License for more details.
 //
 //  =================================================================
 //
-//    24.02.06   <--  Date of Last Modification.
+//    08.07.08   <--  Date of Last Modification.
 //                   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //  -----------------------------------------------------------------
 //
@@ -59,7 +34,7 @@
 //                  CSBStructure ( SB structure (monomer) class )
 //                  CSBIndex     ( SB index class               )
 //
-//    E. Krissinel 2001-2006
+//   (C) E. Krissinel 2000-2008
 //
 //  =================================================================
 //
@@ -100,33 +75,35 @@ CSBAtom::CSBAtom ( RPCStream Object ) : CStream(Object)  {
 CSBAtom::~CSBAtom() {}
 
 void CSBAtom::SBAtomInit()  {
-  sca_name  [0]  = char(0);
-  pdb_name  [0]  = char(0);
-  element   [0]  = char(0);
-  energyType[0]  = char(0);
-  x              = -MaxReal;
-  y              = -MaxReal;
-  z              = -MaxReal;
-  x_esd          = 0.0;
-  y_esd          = 0.0;
-  z_esd          = 0.0;
-  ccp4_charge    = 0.0;  // atom charge from ccp4 libs
-  sca_charge     = 0.0;  // formal atom charge (MSD)
-  partial_charge = 0.0;  // partial atom charge (MSD)
-  vdw_radius     = 0.0;
-  vdwh_radius    = 0.0;
-  ion_radius     = 0.0;
-  valency        = 0;
-  chirality      = 'N';
-  leaving        = 'N';
-  hb_type        = 'N';
+  sca_name    [0] = char(0);
+  pdb_name    [0] = char(0);
+  old_pdb_name[0] = char(0);
+  element     [0] = char(0);
+  energyType  [0] = char(0);
+  x               = -MaxReal;
+  y               = -MaxReal;
+  z               = -MaxReal;
+  x_esd           = 0.0;
+  y_esd           = 0.0;
+  z_esd           = 0.0;
+  ccp4_charge     = 0.0;  // atom charge from ccp4 libs
+  sca_charge      = 0.0;  // formal atom charge (MSD)
+  partial_charge  = 0.0;  // partial atom charge (MSD)
+  vdw_radius      = 0.0;
+  vdwh_radius     = 0.0;
+  ion_radius      = 0.0;
+  valency         = 0;
+  chirality       = 'N';
+  leaving         = 'N';
+  hb_type         = 'N';
 }
 
 void CSBAtom::Copy ( PCSBAtom A )  {
-  strcpy ( sca_name  ,A->sca_name   );
-  strcpy ( pdb_name  ,A->pdb_name   );
-  strcpy ( element   ,A->element    );
-  strcpy ( energyType,A->energyType );
+  strcpy ( sca_name    ,A->sca_name     );
+  strcpy ( pdb_name    ,A->pdb_name     );
+  strcpy ( old_pdb_name,A->old_pdb_name );
+  strcpy ( element     ,A->element      );
+  strcpy ( energyType  ,A->energyType   );
   x              = A->x;
   y              = A->y;
   z              = A->z;
@@ -167,12 +144,13 @@ PCAtom a;
 
 
 void CSBAtom::write ( RCFile f )  {
-int Version=4;
+int Version=5;
   f.WriteInt  ( &Version                      );
-  f.WriteFile ( sca_name  ,sizeof(sca_name)   );
-  f.WriteFile ( pdb_name  ,sizeof(pdb_name)   );
-  f.WriteFile ( element   ,sizeof(element)    );
-  f.WriteFile ( energyType,sizeof(energyType) );
+  f.WriteFile ( sca_name    ,sizeof(sca_name)     );
+  f.WriteFile ( pdb_name    ,sizeof(pdb_name)     );
+  f.WriteFile ( old_pdb_name,sizeof(old_pdb_name) );
+  f.WriteFile ( element     ,sizeof(element)      );
+  f.WriteFile ( energyType  ,sizeof(energyType)   );
   f.WriteReal ( &x                            );
   f.WriteReal ( &y                            );
   f.WriteReal ( &z                            );
@@ -196,6 +174,9 @@ int Version;
   f.ReadInt  ( &Version                      );
   f.ReadFile ( sca_name  ,sizeof(sca_name)   );
   f.ReadFile ( pdb_name  ,sizeof(pdb_name)   );
+  if (Version>4)
+    f.ReadFile ( old_pdb_name,sizeof(old_pdb_name) );
+  else  strcpy ( old_pdb_name,pdb_name );
   f.ReadFile ( element   ,sizeof(element)    );
   f.ReadFile ( energyType,sizeof(energyType) );
   f.ReadReal ( &x                            );
@@ -493,19 +474,19 @@ void CSBStructure::SBStructureInit()  {
 
 }
 
-void CSBStructure::PutFormula ( const pstr F )  {
+void CSBStructure::PutFormula ( cpstr F )  {
   CreateCopy ( Formula,F );
 }
 
-void CSBStructure::PutName ( const pstr N )  {
+void CSBStructure::PutName ( cpstr N )  {
   CreateCopy ( Name,N );
 }
 
-void CSBStructure::PutSynonym ( const pstr S )  {
+void CSBStructure::PutSynonym ( cpstr S )  {
   CreateCopy ( Synonym,S );
 }
 
-void CSBStructure::PutCharge ( const pstr G )  {
+void CSBStructure::PutCharge ( cpstr G )  {
   CreateCopy ( Charge,G );
 }
 
@@ -615,8 +596,8 @@ int i;
     if (Atom[i])  Atom[i]->energyType[0] = char(0);
 }
 
-int  CSBStructure::SetEnergyType ( const pstr sca_name,
-                                   const pstr energyType,
+int  CSBStructure::SetEnergyType ( cpstr sca_name,
+                                   cpstr energyType,
                                    realtype partial_charge )  {
 int n;
   n = GetAtomNo ( sca_name );
@@ -627,7 +608,7 @@ int n;
   return n;
 }
 
-int CSBStructure::GetAtomNo ( const pstr sca_name )  {
+int CSBStructure::GetAtomNo ( cpstr sca_name )  {
 int n,i;
   n = 0;
   for (i=0;(i<nAtoms) && (!n);i++)
@@ -638,7 +619,7 @@ int n,i;
   return n;
 }
 
-PCSBAtom CSBStructure::GetAtom ( const pstr sca_name )  {
+PCSBAtom CSBStructure::GetAtom ( cpstr sca_name )  {
 int n;
   n = GetAtomNo ( sca_name );
   if (n>0)  return Atom[n-1];
@@ -1311,7 +1292,7 @@ int i;
   ldStructure = NULL;
 }
 
-pstr CSBase0::GetPath ( pstr & S, const pstr FName )  {
+pstr CSBase0::GetPath ( pstr & S, cpstr FName )  {
   if (dirpath)  {
     if (S)  delete[] S;
     S = new char[strlen(dirpath)+strlen(FName)+10];
@@ -1322,11 +1303,11 @@ pstr CSBase0::GetPath ( pstr & S, const pstr FName )  {
   return S;
 }
 
-int CSBase0::LoadIndex1 ( const pstr EnvVar )  {
+int CSBase0::LoadIndex1 ( cpstr EnvVar )  {
   return LoadIndex ( getenv(EnvVar) );
 }
 
-int CSBase0::LoadIndex ( const pstr path )  {
+int CSBase0::LoadIndex ( cpstr path )  {
 CFile      f;
 pstr       S;
 int        i;
@@ -1375,7 +1356,7 @@ PPCSBIndex Index1;
 }
 
 
-int CSBase0::LoadStructure ( const pstr compoundID )  {
+int CSBase0::LoadStructure ( cpstr compoundID )  {
 //   LoadStructure(..) reads structure from *.sbase files and
 // stores it in RAM for faster access. There are no special
 // functions to access loaded structures, all requests to
@@ -1454,8 +1435,8 @@ int            structNo,i,k,RC;
 
 }
 
-int  CSBase0::UnloadStructure ( const pstr compoundID )  {
-//   UnloadStructure(..) deletes structure from RAM and releases
+int  CSBase0::UnloadStructure ( cpstr compoundID )  {
+//   UnloadStructure(..) deletes strtucture from RAM and releases
 // its memory. The structure is then accessible through a normal
 // way from *.sbase files, which is slower.
 int  structNo,ldPos;
@@ -1502,7 +1483,7 @@ int MakeElementType ( int ElType, char chirality, Boolean Cflag )  {
 }
 
 
-int CSBase0::GetStructNo ( const pstr compoundID )  {
+int CSBase0::GetStructNo ( cpstr compoundID )  {
 int  l1,l2,l,k;
 char id[20];
   strcpy_css ( id,compoundID );
@@ -1556,7 +1537,7 @@ pstr   S;
 }
 
 
-PCSBStructure CSBase0::GetStructure ( const pstr compoundID )  {
+PCSBStructure CSBase0::GetStructure ( cpstr compoundID )  {
 //   GetStructure returns pointer to the monomer structure
 // identified by 3-letter compoundID. If such structure is not
 // found, the function returns NULL.
@@ -1610,7 +1591,7 @@ PCSBStructure SBS;
   return SBS;
 }
 
-PCSBStructure CSBase0::GetStructure ( const pstr compoundID,
+PCSBStructure CSBase0::GetStructure ( cpstr compoundID,
                                       PCFile structFile )  {
 //   Another form of GetStructure(..) uses an open structure
 // file, which allows to save on opening/closing file if
@@ -1638,7 +1619,7 @@ int           structNo;
 }
 
 
-PCResidue CSBase0::makeCResidue ( const pstr compoundID, 
+PCResidue CSBase0::makeCResidue ( cpstr compoundID, 
                                   PCFile     structFile,
                                   Boolean    includeHydrogens,
                                   Boolean    makeTer )  {
@@ -1675,7 +1656,7 @@ int  CSBase0::GetNofAtoms ( int  structNo )  {
   else  return SBASE_StructNotFound;
 }
 
-int  CSBase0::GetNofAtoms ( const pstr compoundID )  {
+int  CSBase0::GetNofAtoms ( cpstr compoundID )  {
 int  structNo;
   structNo = GetStructNo ( compoundID );
   if (structNo!=SBASE_StructNotFound)
@@ -1771,10 +1752,47 @@ int rc,htype;
 
 int  CSBase0::GetGraph ( int structNo, RPCGraph G, int Hflag )  {
 PCFile graphFile;
-int    rc;
+int    htype;
+
+  if ((0<=structNo) && (structNo<nStructures))  {
+    if (Index[structNo]->loadPos>=0)  {
+      if (!G) G = new CGraph();
+      G->Copy ( ldGraph[Index[structNo]->loadPos] );
+    } else  {
+      graphFile = GetGraphFile();
+      if (graphFile)  {
+        graphFile->seek ( Index[structNo]->fGraphPos );
+        StreamRead  ( *graphFile,G );
+        graphFile->shut ();
+        delete graphFile;
+      } else  {
+        if (G)  delete G;
+        G = NULL;
+        return SBASE_FileNotFound;
+      }
+    }
+  } else  {
+    if (G)  delete G;
+    G = NULL;
+    return SBASE_WrongIndex;
+  }
+
+  if (G)  {
+    G->MakeVertexIDs();
+    if (Hflag>=1)  {
+      htype = getElementNo(pstr("H"));
+      if (Hflag==2)  G->HideType    ( htype );
+               else  G->ExcludeType ( htype );
+    }
+    G->Build ( False );
+  }
+
+  return SBASE_Ok;
+
+  /*
   graphFile = GetGraphFile();
   if (graphFile)  {
-    rc = GetGraph ( structNo,G,Hflag );
+    rc = GetGraph ( graphFile,G,Hflag );
     graphFile->shut();
     delete graphFile;
     return rc;
@@ -1783,11 +1801,49 @@ int    rc;
     G = NULL;
     return SBASE_FileNotFound;
   }
+  */
+
 }
 
-int  CSBase0::GetGraph ( const pstr compoundID, RPCGraph G,
+int  CSBase0::GetGraph ( cpstr compoundID, RPCGraph G,
                          int Hflag )  {
-int structNo = GetStructNo ( compoundID );
+PCFile graphFile;
+int    structNo,htype;
+
+  structNo = GetStructNo ( compoundID );
+
+  if (structNo!=SBASE_StructNotFound)  {
+    if (Index[structNo]->loadPos>=0)  {
+      if (!G )  G = new CGraph();
+      G->Copy ( ldGraph[Index[structNo]->loadPos] );
+    } else  {
+      graphFile = GetGraphFile();
+      if (graphFile)  {
+        graphFile->seek ( Index[structNo]->fGraphPos );
+        StreamRead      ( *graphFile,G );
+        graphFile->shut ();
+        delete graphFile;
+      } else  {
+        if (G)  delete G;
+        G = NULL;
+        return SBASE_FileNotFound;
+      }
+    }
+  }
+
+  if (G)  {
+    G->MakeVertexIDs();
+    if (Hflag>=1)  {
+      htype = getElementNo(pstr("H"));
+      if (Hflag==2)  G->HideType    ( htype );
+               else  G->ExcludeType ( htype );
+    }
+    G->Build ( False );
+  }
+
+  return  structNo;
+
+  /*
   if (structNo!=SBASE_StructNotFound)  {
     return  GetGraph ( structNo,G,Hflag );
   } else  {
@@ -1795,6 +1851,8 @@ int structNo = GetStructNo ( compoundID );
     G = NULL;
     return  structNo;
   }
+  */
+
 }
 
 int  CSBase0::CheckGraph ( PCGraph G, int Hflag,
@@ -1906,7 +1964,7 @@ int CSBase0::CheckResidue ( PCResidue R,
                             int Hflag, Boolean Cflag,
                             int & nInResidue, int & nInStructure,
                             int & nMatched,   ivector match,
-                            const pstr altLoc, int minMatchSize )  {
+                            cpstr altLoc, int minMatchSize )  {
 PCGraph G;
 int     rc,htype;
 
@@ -2800,7 +2858,7 @@ pstr  p1,p2;
 
 }
 
-int CSBase0::GetAtoms ( const pstr name,
+int CSBase0::GetAtoms ( cpstr name,
                         int & nNonHAtoms, PAtomName NonHAtName,
                         int & nHAtoms,    PAtomName HAtName,
                         ivector Hconnect, ivector Elem,
@@ -2893,7 +2951,7 @@ PCFile        structFile;
 }
 
 
-int  CSBase0::GetBonds ( const pstr    name,
+int  CSBase0::GetBonds ( cpstr    name,
                          ivector nBonds, imatrix bondPair,
                          int &   nAtoms, int     maxNAtoms,
                          int     maxNBonds )  {
@@ -2959,7 +3017,7 @@ int     structNo,i, a1,a2;
 }
 
 
-int  CSBase0::GetHetInfo ( const pstr  name,
+int  CSBase0::GetHetInfo ( cpstr       name,
                            pstr        Formula,
                            pstr        Hname,
                            pstr        Hsynonym,
