@@ -22,7 +22,7 @@
 //
 //  =================================================================
 //
-//    08.07.08   <--  Date of Last Modification.
+//    04.02.09   <--  Date of Last Modification.
 //                   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //  -----------------------------------------------------------------
 //
@@ -40,7 +40,7 @@
 //                   RName1 ( 1-characters amino acid names         )
 //
 //
-//  (C) E. Krissinel  2000-2008
+//  (C) E. Krissinel  2000-2009
 //
 //  =================================================================
 //
@@ -603,6 +603,17 @@ int     i;
   return isThere;
 }
 
+int isDNARNA ( cpstr resName )  {
+Boolean isThere;
+int     i;
+  isThere = False;
+  for (i=0;(i<nNucleotideNames) && (!isThere);i++)
+    isThere = (!strcmp(NucleotideName[i],resName));
+  if (!isThere)         return 0;  // neither
+  if (resName[0]=='D')  return 1;  // DNA
+  return 2;                        // RNA
+}
+
 Boolean isSugar ( cpstr resName )  {
   return False;
 }
