@@ -22,7 +22,7 @@
 //
 //  =================================================================
 //
-//    08.07.08   <--  Date of Last Modification.
+//    17.03.09   <--  Date of Last Modification.
 //                   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //  -----------------------------------------------------------------
 //
@@ -43,9 +43,10 @@
 //                  CSupersede
 //                  CJournal
 //                  CRemark
+//                  CBiomolecule
 //                  CMMDBTitle  ( MMDB title section )
 //
-//  (C) E. Krissinel 2000-2008
+//  (C) E. Krissinel 2000-2009
 //
 //  =================================================================
 //
@@ -1370,6 +1371,415 @@ byte Version;
 MakeStreamFunctions(CRemark)
 
 
+//  =================  CBiomolecule  =====================
+
+#define  R350_ERRBIOMT     (-3)
+#define  R350_ERROR        (-2)
+#define  R350_END          (-1)
+#define  R350_NONE           0
+#define  R350_BIOMOLECULE    1
+#define  R350_CHAINS         2
+#define  R350_BIOMT          3
+
+void getRemarkKey ( RPCRemark rem, int & lkey )  {
+  if (rem)  {
+    if (rem->remarkNum!=350)  lkey = R350_END;
+    else if (rem->Remark)  {
+      if (strcasestr(rem->Remark,"BIOMOLECULE:"))
+        lkey = R350_BIOMOLECULE;
+      else if (strcasestr(rem->Remark,"CHAINS:"))
+        lkey = R350_CHAINS;
+      else if (strcasestr(rem->Remark,"BIOMT1") ||
+               strcasestr(rem->Remark,"BIOMT2") ||
+               strcasestr(rem->Remark,"BIOMT3"))
+        lkey = R350_BIOMT;
+      else
+        lkey = R350_NONE;
+    }
+  }
+}
+
+int lookupRemarks ( int & i, RPCRemark rem,
+                    RCTitleContainer Remark )  {
+int l,lkey;
+
+  l    = Remark.Length();
+  lkey = R350_NONE;
+  while ((i<l) && (lkey==R350_NONE))  {
+    getRemarkKey ( rem,lkey );
+    if (lkey==R350_NONE)  {
+      i++;
+      rem = (PCRemark)Remark.GetContainerClass ( i );
+    }
+  }
+
+  return lkey;
+
+}
+
+
+
+CBMApply::CBMApply() : CStream()  {
+  InitBMApply();
+}
+
+CBMApply::CBMApply ( RPCStream Object ) : CStream ( Object )  {
+  InitBMApply();
+}
+
+CBMApply::~CBMApply()  {
+  FreeMemory();
+}
+
+void  CBMApply::InitBMApply()  {
+  chain     = NULL;
+  nChains   = 0;
+  tm        = NULL;
+  nMatrices = 0;
+}
+
+void  CBMApply::FreeMemory()  {
+  if (chain)  delete[] chain;
+  if (tm)     delete[] tm;
+  chain     = NULL;
+  nChains   = 0;
+  tm        = NULL;
+  nMatrices = 0;
+}
+
+int  CBMApply::addChains ( int & i, RPCRemark rem,
+                           RCTitleContainer Remark )  {
+PChainID ch1;
+pstr     p;
+int      l,lkey,nAdd,j;
+
+  l    = Remark.Length();
+  lkey = R350_NONE;
+
+  while ((i<l) && (lkey==R350_NONE))  {
+
+    p = strcasestr ( rem->Remark,"CHAINS:" );
+    if (p)  p += 7;
+    else  {
+      p = rem->Remark;
+      while (*p==' ')  p++;
+      if ((p[1]!=',') && (p[1]!=' '))  p = NULL;
+    }
+ 
+    if (p)  {
+      nAdd  = strlen(p)/2 + 3;
+      ch1   = chain;
+      chain = new ChainID[nChains+nAdd];
+      for (j=0;j<nChains;j++)
+        strcpy ( chain[j],ch1[j] );
+      if (ch1)  delete[] ch1;
+
+      while (*p)  {
+        while ((*p==' ') || (*p==','))  p++;
+        if (*p)  {
+          if ((p[1]==',') || (p[1]==' ') || (p[1]==char(0)))  {
+            chain[nChains][0] = *p;
+            chain[nChains][1] = char(0);
+            nChains++;
+            p++;
+          } else
+            break;
+        }
+      }
+    }
+
+    do  {
+      i++;
+      if (i<l)  {
+        rem = (PCRemark)Remark.GetContainerClass ( i );
+        if (rem)  {
+          if (rem->remarkNum!=350)  lkey = R350_END;
+          else getRemarkKey ( rem,lkey );
+        }
+      } else
+        lkey = R350_END;
+    } while ((!rem) && (lkey==R350_NONE));
+
+  }
+
+  return lkey;
+
+}
+
+int getBIOMT ( RPCRemark rem, int biomtNo, mat44 & t,
+               RCTitleContainer Remark, int & i )  {
+char PN[20];
+pstr p1,p2;
+int  l,j,lkey;
+
+  sprintf ( PN,"BIOMT%1i",biomtNo );
+  p1 = strcasestr ( rem->Remark,PN );
+  if (!p1)  return R350_ERRBIOMT;
+
+  p1 += 6;
+  while (*p1==' ')  p1++;
+  while (*p1 && (*p1!=' '))  p1++;
+
+  l = biomtNo - 1;
+  t[l][0] = strtod ( p1,&p2 );
+  if (p1==p2)  return R350_ERRBIOMT;
+  t[l][1] = strtod ( p2,&p1 );
+  if (p1==p2)  return R350_ERRBIOMT;
+  t[l][2] = strtod ( p1,&p2 );
+  if (p1==p2)  return R350_ERRBIOMT;
+  t[l][3] = strtod ( p2,&p1 );
+  if (p1==p2)  return R350_ERRBIOMT;
+
+  if (biomtNo==3)  {
+    for (j=0;j<3;j++)
+      t[3][j] = 0.0;
+    t[3][3] = 1.0;
+  }
+
+  l    = Remark.Length();
+  lkey = R350_BIOMT;
+  do  {
+    i++;
+    if (i<l)  {
+      rem = (PCRemark)Remark.GetContainerClass ( i );
+      if (rem)  {
+        if (rem->remarkNum!=350)  lkey = R350_END;
+                            else  getRemarkKey ( rem,lkey );
+      }
+    } else
+      lkey = R350_END;
+  } while ((lkey==R350_NONE) || ((!rem) && (lkey==R350_BIOMT)));
+
+  return lkey;
+
+}
+
+int  CBMApply::addMatrices ( int & i, RPCRemark rem,
+                             RCTitleContainer Remark )  {
+pmat44 tm1;
+int    l,lkey,nAdd,j,k1,k2,nAlloc;
+
+  l      = Remark.Length();
+  lkey   = R350_BIOMT;
+  nAlloc = nMatrices;
+
+  while ((i<l) && (lkey==R350_BIOMT))  {
+
+    if (nMatrices>=nAlloc)  {
+      nAlloc = nMatrices + 10;
+      tm1    = tm;
+      tm     = new mat44[nAlloc];
+      for (j=0;j<nMatrices;j++)
+        for (k1=0;k1<4;k1++)
+          for (k2=0;k2<4;k2++)
+            tm[j][k1][k2] = tm1[j][k1][k2];
+      if (tm1)  delete[] tm1;
+    }
+
+    lkey = getBIOMT ( rem,1,tm[nMatrices],Remark,i );
+    if (lkey==R350_BIOMT)
+      lkey = getBIOMT ( rem,2,tm[nMatrices],Remark,i );
+    if (lkey==R350_BIOMT)
+      lkey = getBIOMT ( rem,3,tm[nMatrices],Remark,i );
+    nMatrices++;
+
+  }
+
+  return lkey;
+
+}
+
+void  CBMApply::Copy ( PCBMApply BMA )  {
+// if BMA is NULL, then empties the class
+int  i,j,k;
+
+  FreeMemory();
+
+  if (BMA)  {
+
+    nChains = BMA->nChains;
+    if (nChains>0)  {
+      chain = new ChainID[nChains];
+      for (i=0;i<nChains;i++)
+        strcpy ( chain[i],BMA->chain[i] );
+    }
+
+    nMatrices = BMA->nMatrices;
+    if (nMatrices>0)  {
+      tm = new mat44[nMatrices];
+      for (i=0;i<nMatrices;i++)
+        for (j=0;j<4;j++)
+          for (k=0;k<4;k++)
+            tm[i][j][k] = BMA->tm[i][j][k];
+     }
+  }
+
+}
+
+void  CBMApply::write ( RCFile f )  {
+int i,j,k;
+  f.WriteInt ( &nChains );
+  for (i=0;i<nChains;i++)
+    f.WriteTerLine ( chain[i],False );
+  f.WriteInt ( &nMatrices );
+  for (i=0;i<nMatrices;i++)
+    for (j=0;j<3;j++)
+      for (k=0;k<4;k++)
+        f.WriteReal ( &(tm[i][j][k]) );
+}
+
+void  CBMApply::read ( RCFile f )  {
+int i,j,k;
+  FreeMemory();
+  f.ReadInt ( &nChains );
+  if (nChains>0)  {
+    chain = new ChainID[nChains];
+    for (i=0;i<nChains;i++)
+      f.ReadTerLine ( chain[i],False );
+  }
+  f.ReadInt ( &nMatrices );
+  if (nMatrices>0)  {
+    tm = new mat44[nMatrices];
+    for (i=0;i<nMatrices;i++)  {
+      for (j=0;j<3;j++)  {
+        for (k=0;k<4;k++)
+          f.ReadReal ( &(tm[i][j][k]) );
+        tm[i][3][j] = 0.0;
+      }
+      tm[i][3][3] = 1.0;
+    }
+  }
+}
+
+MakeStreamFunctions(CBMApply)
+
+
+CBiomolecule::CBiomolecule() : CStream()  {
+  InitBiomolecule();
+}
+
+CBiomolecule::CBiomolecule ( RPCStream Object )
+            : CStream ( Object )  {
+  InitBiomolecule();
+}
+
+CBiomolecule::~CBiomolecule()  {
+  FreeMemory();
+}
+
+void  CBiomolecule::InitBiomolecule()  {
+  BMApply = NULL;
+  nBMAs   = 0;
+}
+
+void  CBiomolecule::FreeMemory()  {
+int i;
+  if (BMApply)  {
+    for (i=0;i<nBMAs;i++)
+      if (BMApply[i])  delete BMApply[i];
+    delete[] BMApply;
+    BMApply = NULL;
+  }
+  nBMAs = 0;
+}
+
+
+PCBMApply CBiomolecule::addBMApply()  {
+PPCBMApply BMA1;
+int        i;
+  BMA1 = BMApply;
+  BMApply = new PCBMApply[nBMAs+1];
+  for (i=0;i<nBMAs;i++)
+    BMApply[i] = BMA1[i];
+  if (BMA1)  delete[] BMA1;
+  BMApply[nBMAs] = new CBMApply();
+  nBMAs++;
+  return BMApply[nBMAs-1];
+}
+
+int CBiomolecule::Size()  {
+int i,k;
+  k = 0;
+  for (i=0;i<nBMAs;i++)
+    k += BMApply[i]->nChains*BMApply[i]->nMatrices;
+  return k;
+}
+
+Boolean CBiomolecule::checkComposition ( PChainID chID, ivector occ,
+                                         ivector  wocc, int n )  {
+// chID[n] is list of chain IDs
+// occ[n]  is list of chain occurencies
+// wocc[n] is working array
+int     i,j,k,k1;
+Boolean cmp;
+
+  for (i=0;i<n;i++)
+    wocc[i] = 0;
+
+  cmp = True;
+
+  for (i=0;(i<nBMAs) && cmp;i++)
+    for (j=0;(j<BMApply[i]->nChains) && cmp;j++)  {
+      k1 = -1;
+      for (k=0;(k<n) && (k1<0);k++)
+        if (!strcmp(chID[k],BMApply[i]->chain[j]))
+          k1 = k;
+      if (k1<0)  cmp = False;  // chain not found in the list
+           else  wocc[k1] += BMApply[i]->nMatrices;
+    }
+
+  for (i=0;(i<n) && cmp;i++)
+    if (occ[i]!=wocc[i])  cmp = False;
+
+  return cmp;
+
+}
+
+void  CBiomolecule::Copy ( PCBiomolecule B )  {
+// if B is NULL, then empties the class
+int  i,j,k;
+
+  FreeMemory();
+
+  if (B)  {
+
+    nBMAs = B->nBMAs;
+    if (nBMAs>0)  {
+      BMApply = new PCBMApply[nBMAs];
+      for (i=0;i<nBMAs;i++)
+        if (B->BMApply[i])  {
+          BMApply[i] = new CBMApply();
+          BMApply[i]->Copy ( B->BMApply[i] );
+        } else
+          BMApply[i] = NULL;
+    }
+
+  }
+
+}
+
+void  CBiomolecule::write ( RCFile f )  {
+int i;
+  f.WriteInt ( &nBMAs );
+  for (i=0;i<nBMAs;i++)
+    StreamWrite ( f,BMApply[i] );
+}
+
+void  CBiomolecule::read ( RCFile f )  {
+int i;
+  FreeMemory();
+  f.ReadInt ( &nBMAs );
+  if (nBMAs>0)  {
+    BMApply = new PCBMApply[nBMAs];
+    for (i=0;i<nBMAs;i++)  {
+      BMApply[i] = NULL;
+      StreamRead ( f,BMApply[i] );
+    }
+  }
+}
+
+MakeStreamFunctions(CBiomolecule)
+
 
 //  =====================   CMMDBFTitle   =======================
 
@@ -1387,18 +1797,23 @@ void  CMMDBTitle::Init()  {
   classification = NULL;
   depDate[0]     = char(0);
   idCode [0]     = char(0);
+  resolution     = -2.0;
   col73          = False;
 
+  Biomolecule    = NULL;
+  nBiomolecules  = 0;
+
 }
 
 CMMDBTitle::~CMMDBTitle() {
-  FreeMemory();
+  FreeMemory ( False );
 }
 
-void  CMMDBTitle::FreeMemory()  {
+void  CMMDBTitle::FreeMemory ( Boolean keepBiomolecules )  {
 
   if (classification)  delete[] classification;
   classification = NULL;
+  resolution     = -2.0;
 
   ObsData  .FreeContainer();
   Title    .FreeContainer();
@@ -1415,8 +1830,23 @@ void  CMMDBTitle::FreeMemory()  {
 
   col73 = False;
 
+  if (!keepBiomolecules)
+    FreeBiomolecules();
+
 }
 
+void  CMMDBTitle::FreeBiomolecules()  {
+int  i;
+  if (Biomolecule)  {
+    for (i=0;i<nBiomolecules;i++)
+      if (Biomolecule[i])  delete Biomolecule[i];
+    delete[] Biomolecule;
+    Biomolecule = NULL;
+  }
+  nBiomolecules = 0;
+}
+
+
 void  CMMDBTitle::SetHeader ( cpstr Classification,
                               cpstr DepDate,
                               cpstr IDCode )  {
@@ -1530,6 +1960,123 @@ PCTitleContainer CMMDBTitle::GetRemarks()  {
   return &Remark;
 }
 
+realtype CMMDBTitle::GetResolution()  {
+//  returns -1.0 if there is no resolution record in the file
+PCRemark rem;
+pstr     p,eptr;
+int      i,l;
+  if (resolution>-1.5)  return resolution;
+  l = Remark.Length();
+  for (i=0;(i<l) && (resolution<-1.5);i++)  {
+    rem = (PCRemark)Remark.GetContainerClass ( i );
+    if (rem)  {
+      if (rem->remarkNum==2)  {
+        if (rem->Remark)  {
+          p = strcasestr ( rem->Remark,"RESOLUTION" );
+          if (p)  {
+            while ((*p) && (*p!=' '))  p++;
+            if (*p)  {
+              resolution = strtod ( p,&eptr );
+              if ((resolution<0.0) || (eptr==p))
+                resolution = -1.0;
+            }
+          }
+        }
+      } else if (rem->remarkNum>2)
+        resolution = -1.0;
+    }
+  }
+  return resolution;
+}
+
+PCBiomolecule CMMDBTitle::addBiomolecule()  {
+PPCBiomolecule  BM1;
+int             i;
+  BM1 = Biomolecule;
+  Biomolecule = new PCBiomolecule[nBiomolecules+1];
+  for (i=0;i<nBiomolecules;i++)
+    Biomolecule[i] = BM1[i];
+  if (BM1)  delete[] BM1;
+  Biomolecule[nBiomolecules] = new CBiomolecule();
+  nBiomolecules++;
+  return Biomolecule[nBiomolecules-1];
+}
+
+int CMMDBTitle::ParseBiomolecules()  {
+PCRemark       rem;
+PCBiomolecule  BMol;
+PCBMApply      BMA;
+pstr           p,eptr;
+int            i,l, lkey;
+
+  FreeBiomolecules();
+
+  l    = Remark.Length();
+  i    = 0;
+  lkey = 0;
+  while ((i<l) && (!lkey))  {
+    rem = (PCRemark)Remark.GetContainerClass ( i );
+    if (rem)  {
+      if (rem->remarkNum==350)      lkey = 1;
+      else if (rem->remarkNum>350)  lkey = -1;
+    }
+    if (!lkey) i++;
+  }
+
+  BMol = NULL;
+  BMA  = NULL;
+
+  while (lkey>0)  {
+
+    rem = (PCRemark)Remark.GetContainerClass ( i );
+    lkey = lookupRemarks ( i,rem,Remark );
+
+    switch (lkey)  {
+      case R350_BIOMOLECULE : BMol = addBiomolecule();
+                              i++;
+                            break;
+      case R350_CHAINS      : if (BMol)  {
+                                BMA = BMol->addBMApply();
+                                while (lkey==R350_CHAINS)
+                                  lkey = BMA->addChains(i,rem,Remark);
+                              } else
+                                lkey = R350_ERROR;
+                            break;
+      case R350_BIOMT       : if (BMA)
+                                lkey = BMA->addMatrices(i,rem,Remark);
+                              else
+                                lkey = R350_ERROR;
+                            break;
+      default : i++;
+    }
+
+  }
+
+  if (lkey<=R350_ERROR)  {
+    FreeBiomolecules();
+    return lkey;
+  }
+
+  return nBiomolecules;
+
+}
+
+int CMMDBTitle::GetNofBiomolecules()  {
+  return nBiomolecules;
+}
+
+void CMMDBTitle::GetBiomolecules ( PPCBiomolecule & BM, int & nBMs ) {
+  BM   = Biomolecule;
+  nBMs = nBiomolecules;
+}
+
+PCBiomolecule CMMDBTitle::GetBiomolecule ( int bmNo )  { // bmno=0,1,..
+  if ((0<=bmNo) && (bmNo<nBiomolecules))
+    return Biomolecule[bmNo];
+  return NULL;
+}
+
+
 int  CMMDBTitle::GetCIF ( PCMMCIFData CIF )  {
 pstr  S;
 int  RC;
@@ -1773,12 +2320,16 @@ char DateCIF[20];
 }
 
 void  CMMDBTitle::Copy ( PCMMDBTitle TS )  {
+int  i;
+
+  FreeBiomolecules();
 
   if (TS)  {
 
     CreateCopy ( classification,TS->classification );
     strcpy     ( depDate       ,TS->depDate        );
     strcpy     ( idCode        ,TS->idCode         );
+    resolution = TS->resolution;
 
     ObsData  .Copy ( &(TS->ObsData)   );
     Title    .Copy ( &(TS->Title)     );
@@ -1793,10 +2344,22 @@ void  CMMDBTitle::Copy ( PCMMDBTitle TS )  {
     Journal  .Copy ( &(TS->Journal)   );
     Remark   .Copy ( &(TS->Remark)    );
 
+    nBiomolecules = TS->nBiomolecules;
+    if (nBiomolecules>0)  {
+      Biomolecule = new PCBiomolecule[nBiomolecules];
+      for (i=0;i<nBiomolecules;i++)
+        if (TS->Biomolecule[i])  {
+          Biomolecule[i] = new CBiomolecule();
+          Biomolecule[i]->Copy ( TS->Biomolecule[i] );
+        } else
+          Biomolecule[i] = NULL;
+    }
+
   } else  {
 
     if (classification)  delete[] classification;
     classification = NULL;
+    resolution     = -2.0;
     ObsData  .FreeContainer();
     Title    .FreeContainer();
     CAVEAT   .FreeContainer();
@@ -1821,14 +2384,16 @@ void  CMMDBTitle::TrimInput ( pstr PDBString )  {
 
 void  CMMDBTitle::write ( RCFile f )  {
 // writes header to PDB binary file
-byte Version=1;
+int  i;
+byte Version=2;
 
-  f.WriteByte    ( &Version );
+  f.WriteByte    ( &Version       );
 
   //  Header data
   f.CreateWrite  ( classification );
   f.WriteTerLine ( depDate,False  );
   f.WriteTerLine ( idCode ,False  );
+  f.WriteReal    ( &resolution    );
 
   ObsData  .write ( f );  //  Obsoletion data
   Title    .write ( f );  //  Title
@@ -1842,11 +2407,16 @@ byte Version=1;
   Supersede.write ( f );  //  Supersede records
   Journal  .write ( f );  //  Journal records
   Remark   .write ( f );  //  Remarks
+
+  f.WriteInt ( &nBiomolecules );
+  for (i=0;i<nBiomolecules;i++)
+    StreamWrite ( f,Biomolecule[i] );
   
 }
 
 void  CMMDBTitle::read ( RCFile f )  {
 // reads header from PDB binary file
+int  i;
 byte Version;
 
   f.ReadByte    ( &Version );
@@ -1855,6 +2425,10 @@ byte Version;
   f.CreateRead  ( classification );
   f.ReadTerLine ( depDate,False  );
   f.ReadTerLine ( idCode ,False  );
+  if (Version>1)
+    f.ReadReal  ( &resolution    );
+  else
+    resolution = -2.0;
 
   ObsData  .read ( f );   //  Obsoletion data
   Title    .read ( f );   //  Title
@@ -1869,6 +2443,18 @@ byte Version;
   Journal  .read ( f );   //  Journal records
   Remark   .read ( f );   //  Remarks
 
+  FreeBiomolecules();
+  if (Version>1)  {
+    f.ReadInt ( &nBiomolecules );
+    if (nBiomolecules>0)  {
+      Biomolecule = new PCBiomolecule[nBiomolecules];
+      for (i=0;i<nBiomolecules;i++)  {
+        Biomolecule[i] = NULL;
+        StreamRead ( f,Biomolecule[i] );
+      }
+    }
+  }
+
 }
 
 MakeStreamFunctions(CMMDBTitle)
