@@ -22,7 +22,7 @@
 //
 //  =================================================================
 //
-//    08.07.08   <--  Date of Last Modification.
+//    17.03.09   <--  Date of Last Modification.
 //                   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //  -----------------------------------------------------------------
 //
@@ -43,9 +43,10 @@
 //                  CSupersede
 //                  CJournal
 //                  CRemark
+//                  CBiomolecule
 //                  CMMDBTitle       ( MMDB title section )
 //
-//   (C) E. Krissinel 2000-2008
+//   (C) E. Krissinel 2000-2009
 //
 //  =================================================================
 //
@@ -519,6 +520,69 @@ class CRemark : public CContainerClass  {
 
 };
 
+//  =================  CBiomolecule  =====================
+
+DefineClass(CBMApply)
+DefineStreamFunctions(CBMApply)
+
+class CBMApply : public CStream  {
+
+  public :
+    PChainID  chain;
+    int       nChains;
+    pmat44    tm;
+    int       nMatrices;
+
+    CBMApply ();
+    CBMApply ( RPCStream Object );
+    ~CBMApply();
+
+    void  FreeMemory();
+
+    int   addChains ( int & i, RPCRemark rem, RCTitleContainer Remark );
+    int addMatrices ( int & i, RPCRemark rem, RCTitleContainer Remark );
+
+    void  Copy  ( PCBMApply BMA );  // if BMA is NULL, then empties
+                                    // the class
+    void  write ( RCFile f );
+    void  read  ( RCFile f );
+
+  protected :
+    void  InitBMApply();
+
+};
+
+
+DefineClass(CBiomolecule)
+DefineStreamFunctions(CBiomolecule)
+
+class CBiomolecule : public CStream  {
+
+  public :
+    PPCBMApply BMApply;
+    int        nBMAs;
+
+    CBiomolecule ();
+    CBiomolecule ( RPCStream Object );
+    ~CBiomolecule();
+
+    void  FreeMemory();
+
+    PCBMApply addBMApply();
+
+    int     Size();
+    Boolean checkComposition ( PChainID chID, ivector occ,
+                               ivector  wocc, int n );
+
+    void  Copy  ( PCBiomolecule B );  // if B is NULL, then empties
+                                      // the class
+    void  write ( RCFile f );
+    void  read  ( RCFile f );
+
+  protected :
+    void  InitBiomolecule();
+
+};
 
 //  =================  CMMDBTitle  =======================
 
@@ -537,7 +601,7 @@ class CMMDBTitle : public CStream  {
     CMMDBTitle ( RPCStream Object );
     ~CMMDBTitle();
 
-    void  FreeMemory();
+    void  FreeMemory ( Boolean keepBiomolecules );
 
     // Fills the PDB file header
     void  SetHeader ( cpstr Classification, // any length is Ok
@@ -564,6 +628,17 @@ class CMMDBTitle : public CStream  {
 
     PCTitleContainer GetRemarks();
 
+    realtype GetResolution(); // -1.0 mean no resolution record in file
+
+    int   ParseBiomolecules(); // returns the number of biomolecules,
+                               // -2 for general format error
+                               // -3 for errors in BIOMT records
+
+    int   GetNofBiomolecules();
+    void  GetBiomolecules   ( PPCBiomolecule & BM, int & nBMs );
+    PCBiomolecule GetBiomolecule ( int bmNo ); // bmno=0,1,..
+                               // returns NULL if bmNo is incorrect
+
     void  PDBASCIIDump ( RCFile      f   );
     void  MakeCIF      ( PCMMCIFData CIF );
 
@@ -584,10 +659,11 @@ class CMMDBTitle : public CStream  {
   protected :
 
     //   Header data
-    pstr    classification;  // classification of the molecule
-    Date    depDate;         // deposition date DD-MMM-YYYY
-    IDCode  idCode;          // unique PDB identifier
-    Boolean col73;           // True if columns 73-80 contain PDB ID
+    pstr     classification;  // classification of the molecule
+    Date     depDate;         // deposition date DD-MMM-YYYY
+    IDCode   idCode;          // unique PDB identifier
+    realtype resolution;      // resolution
+    Boolean  col73;           // True if columns 73-80 contain PDB ID
 
     CTitleContainer ObsData;     // obsoletion data
     CTitleContainer Title;       // title data
@@ -602,7 +678,13 @@ class CMMDBTitle : public CStream  {
     CTitleContainer Journal;     // journal records
     CTitleContainer Remark;      // remark records
 
+    PPCBiomolecule  Biomolecule;
+    int             nBiomolecules;
+
     void  Init();
+    void  FreeBiomolecules();
+
+    PCBiomolecule addBiomolecule();
 
 };
 
