@@ -1,53 +1,28 @@
-//  $Id: mmdb_utils.cpp,v 1.21 2005/12/20 12:08:14 keb Exp $
+//  $Id: mmdb_utils.cpp,v 1.25 2008/07/08 15:28:16 keb Exp $
 //  =================================================================
 //
 //   CCP4 Coordinate Library: support of coordinate-related
 //   functionality in protein crystallography applications.
 //
-//   Copyright (C) Eugene Krissinel 2004.
+//   Copyright (C) Eugene Krissinel 2000-2008.
 //
-//   This library is free software and is distributed under the terms
-//   and conditions of the CCP4 licence agreement as `Part 0' (Annex 2)
-//   software, which is version 2.1 of the GNU Lesser General Public
-//   Licence (LGPL) with the following additional clause:
+//    This library is free software: you can redistribute it and/or 
+//    modify it under the terms of the GNU Lesser General Public 
+//    License version 3, modified in accordance with the provisions 
+//    of the license to address the requirements of UK law.
 //
-//      `You may also combine or link a "work that uses the Library"
-//      to produce a work containing portions of the Library, and
-//      distribute that work under terms of your choice, provided that
-//      you give prominent notice with each copy of the work that the
-//      specified version of the Library is used in it, and that you
-//      include or provide public access to the complete corresponding
-//      machine-readable source code for the Library including whatever
-//      changes were used in the work. (i.e. If you make changes to the
-//      Library you must distribute those, but you do not need to
-//      distribute source or object code to those portions of the work
-//      not covered by this licence.)'
+//    You should have received a copy of the modified GNU Lesser 
+//    General Public License along with this library. If not, copies 
+//    may be downloaded from http://www.ccp4.ac.uk/ccp4license.php
 //
-//   Note that this clause grants an additional right and does not
-//   impose any additional restriction, and so does not affect
-//   compatibility with the GNU General Public Licence (GPL). If you
-//   wish to negotiate other terms, please contact the maintainer.
-//
-//   You can redistribute it and/or modify the library under the terms
-//   of the GNU Lesser General Public License as published by the Free
-//   Software Foundation; either version 2.1 of the License, or (at
-//   your option) any later version.
-//
-//   This library is distributed in the hope that it will be useful,
-//   but WITHOUT ANY WARRANTY; without even the implied warranty of
-//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-//   Lesser General Public License for more details.
-//
-//   You should have received a copy of the CCP4 licence and/or GNU
-//   Lesser General Public License along with this library; if not,
-//   write to the CCP4 Secretary, Daresbury Laboratory, Warrington
-//   WA4 4AD, UK. The GNU Lesser General Public can also be obtained
-//   by writing to the Free Software Foundation, Inc., 59 Temple Place,
-//   Suite 330, Boston, MA 02111-1307 USA
+//    This program is distributed in the hope that it will be useful,
+//    but WITHOUT ANY WARRANTY; without even the implied warranty of
+//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+//    GNU Lesser General Public License for more details.
 //
 //  =================================================================
 //
-//    12.01.06   <--  Date of Last Modification.
+//    08.07.06   <--  Date of Last Modification.
 //                   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //  -----------------------------------------------------------------
 //
@@ -85,7 +60,7 @@
 //                   ParseResID     ( parses residue ID line         )
 //                   ParseAtomPath  ( parses full atom path          )
 //
-//  E. Krissinel  2001-2006
+//   (C) E. Krissinel  2000-2008
 //
 //  =================================================================
 //
@@ -106,20 +81,24 @@
 #include "mmdb_utils.h"
 #endif
 
+#ifndef IOTBX_PDB_HYBRID_36_C_H
+#include "hybrid_36.h"
+#endif
+
 
 // ====================== Date functions  =======================
 
-static pstr Month[12] = {
+static cpstr Month[12] = {
   "JAN", "FEB", "MAR", "APR", "MAY", "JUN",
   "JUL", "AUG", "SEP", "OCT", "NOV", "DEC"
 };
 
-static pstr nMonth[12] = {
+static cpstr nMonth[12] = {
   "01", "02", "03", "04", "05", "06",
   "07", "08", "09", "10", "11", "12"
 };
 
-void  Date9to11 ( const pstr Date9, pstr Date11 )  {
+void  Date9to11 ( cpstr Date9, pstr Date11 )  {
 // converts  DD-MMM-YY to DD-MMM-YYYY
 int i;
   i = 0;
@@ -147,7 +126,7 @@ int i;
   Date11[11] = char(0);
 }
 
-void  Date11to9 ( const pstr Date11, pstr Date9 )  {
+void  Date11to9 ( cpstr Date11, pstr Date9 )  {
 // converts DD-MMM-YYYY to DD-MMM-YY
 int i;
   i = 0;
@@ -170,7 +149,7 @@ int i;
   Date9[6] = '-';
 }
 
-void  Date9toCIF ( const pstr Date9, pstr DateCIF )  {
+void  Date9toCIF ( cpstr Date9, pstr DateCIF )  {
 //  DD-MMM-YY -> YYYY-MM-DD             )
 int  i;
   i = 0;
@@ -192,7 +171,7 @@ int  i;
   DateCIF[10] = char(0);
 }
 
-void  Date11toCIF ( const pstr Date11, pstr DateCIF )  {
+void  Date11toCIF ( cpstr Date11, pstr DateCIF )  {
 //  DD-MMM-YYYY -> YYYY-MM-DD
 int  i;
   i = 0;
@@ -210,7 +189,7 @@ int  i;
   DateCIF[10] = char(0);
 }
 
-void  DateCIFto9 ( const pstr DateCIF, pstr Date9 )  {
+void  DateCIFto9 ( cpstr DateCIF, pstr Date9 )  {
 //  YYYY-MM-DD -> DD-MMM-YY
 int  i;
   strncpy ( Date9,&(DateCIF[8]),2 );
@@ -224,10 +203,10 @@ int  i;
   }
   Date9[6] = '-';
   strncpy ( &(Date9[7]),&(DateCIF[2]),2 );
-  DateCIF[9] = char(0);
+//  DateCIF[9] = char(0);
 }
 
-void  DateCIFto11 ( const pstr DateCIF, pstr Date11 )  {
+void  DateCIFto11 ( cpstr DateCIF, pstr Date11 )  {
 //  YYYY-MM-DD  -> DD-MMM-YYYY
 int  i;
   strncpy ( Date11,&(DateCIF[8]),2 );
@@ -241,13 +220,13 @@ int  i;
   }
   Date11[6] = '-';
   strncpy ( &(Date11[7]),DateCIF,4 );
-  DateCIF[11] = char(0);
+//  DateCIF[11] = char(0);
 }
 
 
 //  =============== Format functions  ===================
 
-Boolean GetInteger ( int & N, const pstr S, int M )  {
+Boolean GetInteger ( int & N, cpstr S, int M )  {
 //   Returns True if S contains an integer number in its
 // first M characters. This number is returned in N.
 //   The return is False if no integer number may be
@@ -264,7 +243,7 @@ char L[50];
     return True;
 }
 
-Boolean GetReal ( realtype & R, const pstr S, int M )  {
+Boolean GetReal ( realtype & R, cpstr S, int M )  {
 //   Returns True if S contains a real number in its
 // first M characters. This number is returned in R.
 //   The return is False if no real number may be
@@ -281,7 +260,7 @@ char L[50];
     return True;
 }
 
-Boolean  GetIntIns ( int & N, pstr ins, const pstr S, int M )  {
+Boolean  GetIntIns ( int & N, pstr ins, cpstr S, int M )  {
 //   Returns True if S contains an integer number in its
 // first M characters. This number is returned in N. In addition
 // to that, GetIntIns() retrieves the insertion code which may
@@ -292,20 +271,28 @@ Boolean  GetIntIns ( int & N, pstr ins, const pstr S, int M )  {
 // "ins" just returns (M+1)th symbol of S (+terminating 0).
 pstr endptr;
 char L[50];
+
   if (S[M]!=' ')  {
     ins[0] = S[M];
     ins[1] = char(0);
   } else
     ins[0] = char(0);
+
   strncpy ( L,S,M );
-  L[M]   = char(0);
-  endptr = NULL;
-  N      = mround(strtod(L,&endptr));
-  if ((N==0) && (endptr==L))  {
-    N = MinInt4;  // no number
-    return False;
-  } else
-    return True;
+  L[M] = char(0);
+  if ((M==4) && ((S[0]>='A') || ((S[0]=='-') && (S[1]>='A'))))
+    hy36decode ( M,L,M,&N);
+  else  {
+    endptr = NULL;
+    N      = mround(strtod(L,&endptr));
+    if ((N==0) && (endptr==L))  {
+      N = MinInt4;  // no number
+      return False;
+    }
+  }
+
+  return True;
+
 }
 
 void  PutInteger ( pstr S, int N, int M )  {
@@ -344,7 +331,7 @@ char N[50];
 }
 
 
-int CIFGetInteger ( int & I, PCMMCIFLoop Loop, const pstr Tag,
+int CIFGetInteger ( int & I, PCMMCIFLoop Loop, cpstr Tag,
                     int & Signal )  {
 int  RC;
 pstr F;
@@ -375,7 +362,7 @@ pstr F;
 }
 
 
-int CIFGetInteger1 ( int & I, PCMMCIFLoop Loop, const pstr Tag,
+int CIFGetInteger1 ( int & I, PCMMCIFLoop Loop, cpstr Tag,
                      int nrow )  {
 int  RC;
 pstr F;
@@ -402,7 +389,7 @@ pstr F;
 }
 
 
-int CIFGetReal ( realtype & R, PCMMCIFLoop Loop, const pstr Tag,
+int CIFGetReal ( realtype & R, PCMMCIFLoop Loop, cpstr Tag,
                  int & Signal )  {
 int  RC;
 pstr F;
@@ -433,7 +420,7 @@ pstr F;
 }
 
 
-int CIFGetReal1 ( realtype & R, PCMMCIFLoop Loop, const pstr Tag,
+int CIFGetReal1 ( realtype & R, PCMMCIFLoop Loop, cpstr Tag,
                   int nrow )  {
 int  RC;
 pstr F;
@@ -460,8 +447,8 @@ pstr F;
 }
 
 
-int  CIFGetString ( pstr S, PCMMCIFLoop Loop, const pstr Tag,
-                    int row, int SLen, const pstr DefS )  {
+int  CIFGetString ( pstr S, PCMMCIFLoop Loop, cpstr Tag,
+                    int row, int SLen, cpstr DefS )  {
 int RC;
 pstr F;
   F = Loop->GetString ( Tag,row,RC );
@@ -476,7 +463,7 @@ pstr F;
 }
 
 
-int CIFGetInteger ( int & I, PCMMCIFStruct Struct, const pstr Tag,
+int CIFGetInteger ( int & I, PCMMCIFStruct Struct, cpstr Tag,
                     Boolean Remove )  {
 int  RC;
 pstr F;
@@ -500,7 +487,7 @@ pstr F;
   return 0;
 }
 
-int CIFGetReal ( realtype & R, PCMMCIFStruct Struct, const pstr Tag,
+int CIFGetReal ( realtype & R, PCMMCIFStruct Struct, cpstr Tag,
                  Boolean Remove )  {
 int RC;
 pstr F;
@@ -524,8 +511,8 @@ pstr F;
   return 0;
 }
 
-int  CIFGetString ( pstr S, PCMMCIFStruct Struct, const pstr Tag,
-                    int SLen, const pstr DefS, Boolean Remove )  {
+int  CIFGetString ( pstr S, PCMMCIFStruct Struct, cpstr Tag,
+                    int SLen, cpstr DefS, Boolean Remove )  {
 int RC;
 pstr F;
   F = Struct->GetString ( Tag,RC );
@@ -540,7 +527,7 @@ pstr F;
 }
 
 
-void  PutIntIns ( pstr S, int N, int M, const pstr ins )  {
+void  PutIntIns ( pstr S, int N, int M, cpstr ins )  {
 //  Integer N is converted into ASCII string of length M
 // and pasted onto first M characters of string S. No
 // terminating zero is added. The insert code ins is put
@@ -550,14 +537,18 @@ void  PutIntIns ( pstr S, int N, int M, const pstr ins )  {
 // appended.
 int  i;
 char L[50];
-  if (N==MinInt4)
+
+  if (N==MinInt4)  {
     for (i=0;i<=M;i++)
       S[i] = ' ';
-  else  { 
-    sprintf   ( L,"%*i",M,N );
+  } else  {
+    if ((M!=4) || ((N>=-999) && (N<=9999)))
+         sprintf    ( L,"%*i",M,N );
+    else hy36encode ( M,N,L );
     strcpy_n1 ( S,L,M );
     if (ins[0]) S[M] = ins[0];
   }
+
 }
 
 
@@ -724,6 +715,7 @@ Boolean B;
     for (j=0;(j<k) && B;j++)
       if (i==j)  B = (fabs(1.0-A[i][j])<eps);
            else  B = (fabs(A[i][j])<eps);
+
   return B;
 
 }
@@ -801,76 +793,86 @@ realtype TransformZ ( mat44 & T, realtype X, realtype Y,
 
 char CIFErrorLocation[200] = "no error";
 
-static pstr msWrongSection =
+static cpstr msWrongSection =
   "Wrong section. The sections in PDB file may be put in wrong order.";
-static pstr msWrongChainID =
+static cpstr msWrongChainID =
   "Wrong chain ID. The input may have changed to another chain.";
-static pstr msWrongEntryID =
+static cpstr msWrongEntryID =
   "Entry ID does not match the header.";
 
-static pstr msSEQRES_serNum   =
+static cpstr msSEQRES_serNum   =
   "Serial numbers of SEQRES records do not increment by 1.";
-static pstr msSEQRES_numRes   =
+static cpstr msSEQRES_numRes   =
   "Different SEQRES records show different numbers of residues.";
-static pstr msSEQRES_extraRes =
+static cpstr msSEQRES_extraRes =
   "SEQRES records contain more residues than specified.";
 
-static pstr msNCSM_Unrecognized =
+static cpstr msNCSM_Unrecognized =
   "Unrecognized numerical input in MTRIXn.";
-static pstr msNCSM_AlreadySet   =
+static cpstr msNCSM_AlreadySet   =
   "Duplicate MTRIXn record.";
-static pstr msNCSM_WrongSerial  =
+static cpstr msNCSM_WrongSerial  =
   "Serial number in MTRIXn record is wrong.";
-static pstr msNCSM_UnmatchIG    =
+static cpstr msNCSM_UnmatchIG    =
   "Different MTRIXn record show different iGiven flag.";
 
-static pstr msATOM_Unrecognized =
+static cpstr msATOM_Unrecognized =
   "Numerical information in ATOM record is not recognized.";
-static pstr msATOM_AlreadySet   =
+static cpstr msATOM_AlreadySet   =
   "Atom is already in the system.";
-static pstr msATOM_NoResidue    =
+static cpstr msATOM_NoResidue    =
   "No residue is found for atom.";
-static pstr msATOM_Unmatch      =
+static cpstr msATOM_Unmatch      =
   "Unmatch in different records for the same atom.";
 
-static pstr msCantOpenFile        = "File can not be opened.";
-static pstr msUnrecognizedInteger = "Wrong ASCII format of an integer.";
-static pstr msWrongModelNo        = "Wrong model number.";
-static pstr msDuplicatedModel     = "Duplicate model number.";
-static pstr msNoModel             = "No model defined.";
-static pstr msForeignFile         = "Attempt to read unknown-type file.";
-static pstr msWrongEdition        = "Attempt to read a higher-version file.";
+static cpstr msCantOpenFile        = "File can not be opened.";
+static cpstr msUnrecognizedInteger =
+  "Wrong ASCII format of an integer.";
+static cpstr msWrongModelNo        = "Wrong model number.";
+static cpstr msDuplicatedModel     = "Duplicate model number.";
+static cpstr msNoModel             = "No model defined.";
+static cpstr msForeignFile         =
+  "Attempt to read unknown-type file.";
+static cpstr msWrongEdition        =
+  "Attempt to read a higher-version file.";
 
-static pstr msNoData              = "Expected data field not found.";
-static pstr msUnrecognizedReal    = "Wrong ASCII format of a real.";
-static pstr msNotACIFFile         = "Not a CIF file ('data_' missing).";
-static pstr msUnrecognCIFItems    = "Unrecognized item(s) in CIF file.";
-static pstr msMissingCIFField     = "Expected CIF item(s) missing.";
-static pstr msEmptyCIFLoop        = "Empty CIF loop encountered.";
-static pstr msUnexpEndOfCIF       = "Unexpected end of CIF file.";
-static pstr msMissgCIFLoopField   = "Inconsistent CIF loop.";
-static pstr msNotACIFStructure    = "Wrong use of CIF structure (as a loop?).";
-static pstr msNotACIFLoop         = "Wrong use of CIF loop (as a structure?).";
+static cpstr msNoData              = "Expected data field not found.";
+static cpstr msUnrecognizedReal    = "Wrong ASCII format of a real.";
+static cpstr msNotACIFFile         =
+  "Not a CIF file ('data_' missing).";
+static cpstr msUnrecognCIFItems    =
+  "Unrecognized item(s) in CIF file.";
+static cpstr msMissingCIFField     = "Expected CIF item(s) missing.";
+static cpstr msEmptyCIFLoop        = "Empty CIF loop encountered.";
+static cpstr msUnexpEndOfCIF       = "Unexpected end of CIF file.";
+static cpstr msMissgCIFLoopField   = "Inconsistent CIF loop.";
+static cpstr msNotACIFStructure    =
+  "Wrong use of CIF structure (as a loop?).";
+static cpstr msNotACIFLoop         =
+  "Wrong use of CIF loop (as a structure?).";
 
-static pstr msNoSheetID           = "No Sheet ID on PDB ASCII card.";
-static pstr msWrongSheetID        = "Wrong Sheet ID.";
-static pstr msWrongStrandNo       = "Wrong Strand number on PDB SHEET card.";
+static cpstr msNoSheetID           = "No Sheet ID on PDB ASCII card.";
+static cpstr msWrongSheetID        = "Wrong Sheet ID.";
+static cpstr msWrongStrandNo       =
+  "Wrong Strand number on PDB SHEET card.";
 
-static pstr msWrongNumberOfStrands = "Wrong number of strands in CIF file.";
-static pstr msWrongSheetOrder     = "Incomplete _struct_sheet_order.";
-static pstr msHBondInconsistency  = "Inconsistency in _struct_sheet_hbond.";
+static cpstr msWrongNumberOfStrands =
+  "Wrong number of strands in CIF file.";
+static cpstr msWrongSheetOrder     = "Incomplete _struct_sheet_order.";
+static cpstr msHBondInconsistency  =
+  "Inconsistency in _struct_sheet_hbond.";
 
-static pstr msEmptyResidueName    = 
+static cpstr msEmptyResidueName    = 
         "No residue name on PDB ATOM or TER card.";
-static pstr msDuplicateSeqNum     = 
+static cpstr msDuplicateSeqNum     = 
         "Duplicate sequence number and insertion code.";
 
-static pstr msEmptyFile           = "Non-existent or empty file.";
+static cpstr msEmptyFile           = "Non-existent or empty file.";
 
-static pstr msNoLogicalName       = "Logical file name not found.";
+static cpstr msNoLogicalName       = "Logical file name not found.";
 
 
-pstr  GetErrorDescription ( int ErrorCode )  {
+cpstr  GetErrorDescription ( int ErrorCode )  {
 
   switch (ErrorCode)  {
 
@@ -956,7 +958,7 @@ CContString::CContString() : CContainerClass()  {
   InitString();
 }
 
-CContString::CContString ( const pstr S ) : CContainerClass()  {
+CContString::CContString ( cpstr S ) : CContainerClass()  {
   InitString();
   ConvertPDBASCII ( S );
 }
@@ -978,7 +980,7 @@ void  CContString::InitString()  {
   CIFTag      = NULL;
 }
 
-int  CContString::ConvertPDBASCII ( const pstr S )  {
+int  CContString::ConvertPDBASCII ( cpstr S )  {
   CreateCopy ( Line,S );
   return 0;
 }
@@ -1250,7 +1252,7 @@ CAtomPath::CAtomPath() : CStream()  {
   InitAtomPath();
 }
 
-CAtomPath::CAtomPath ( const pstr ID ) : CStream()  {
+CAtomPath::CAtomPath ( cpstr ID ) : CStream()  {
   InitAtomPath();
   SetPath ( ID );
 }
@@ -1273,7 +1275,7 @@ void CAtomPath::InitAtomPath()  {
   isSet       = 0;
 }
 
-int CAtomPath::SetPath ( const pstr ID )  {
+int CAtomPath::SetPath ( cpstr ID )  {
 //  1. If ID starts with '/':
 //   /mdl/chn/seq(res).i/atm[elm]:a
 //
@@ -1305,7 +1307,7 @@ int  i,k;
 
   isSet = 0;  // clear all bits.
 
-  p = ID;
+  p = pstr(ID);
   while (*p==' ')  p++;
 
   if (!(*p))  return 0;
@@ -1494,7 +1496,7 @@ void CQuickSort::Sort ( void * sortdata, int data_len )  {
 
 //  --------------------------------------------------------
 
-void  takeWord ( pstr & p, pstr wrd, const pstr ter, int l )  {
+void  takeWord ( pstr & p, pstr wrd, cpstr ter, int l )  {
 pstr p1;
 int  i;
   p1 = strpbrk ( p,ter );
@@ -1511,11 +1513,11 @@ int  i;
 }
 
 
-void ParseAtomID ( const pstr ID, AtomName aname, Element elname,
-                                  AltLoc   aloc )  {
+void ParseAtomID ( cpstr ID, AtomName aname, Element elname,
+                             AltLoc   aloc )  {
 pstr p;
 
-  p = ID;
+  p = pstr(ID);
   while (*p==' ')  p++;
 
   strcpy ( aname ,"*" );
@@ -1538,15 +1540,15 @@ pstr p;
 
 }
 
-int ParseResID ( const pstr ID, int & sn, InsCode inscode,
-                                          ResName resname )  {
+int ParseResID ( cpstr ID, int & sn, InsCode inscode,
+                                     ResName resname )  {
 int  RC;
 pstr p,p1;
 char N[100];
 
   RC = 0;
 
-  p = ID;
+  p = pstr(ID);
   while (*p==' ')  p++;
 
   sn = ANY_RES;
@@ -1580,7 +1582,7 @@ char N[100];
 
 }
 
-int ParseAtomPath ( const pstr ID,
+int ParseAtomPath ( cpstr      ID,
                     int &      mdl,
                     ChainID    chn,
                     int &      sn,
@@ -1599,7 +1601,7 @@ Boolean wasRes;
 
   RC = 0;
 
-  p = ID;
+  p = pstr(ID);
   while (*p==' ')  p++;
 
   mdl = 0;
@@ -1692,7 +1694,7 @@ Boolean wasRes;
 
 
 int  ParseSelectionPath (
-             const pstr CID,
+             cpstr   CID,
              int &   iModel,
              pstr    Chains,
              int &   sNum1,
@@ -1716,7 +1718,7 @@ Boolean wasModel,wasChain,wasRes,haveNeg;
   ID = new char[l];
   N  = new char[l];
 
-  p = CID;
+  p  = pstr(CID);
   p1 = ID;
   while (*p)  {
     if (*p!=' ')  {
@@ -1865,15 +1867,15 @@ Boolean wasModel,wasChain,wasRes,haveNeg;
 void MakeSelectionPath (
              pstr       CID,
              int        iModel,
-             const pstr Chains,
+             cpstr      Chains,
              int        sNum1,
              const InsCode ic1,
              int        sNum2,
              const InsCode ic2,
-             const pstr RNames,
-             const pstr ANames,
-             const pstr Elements,
-             const pstr altLocs
+             cpstr      RNames,
+             cpstr      ANames,
+             cpstr      Elements,
+             cpstr      altLocs
                         )  {
 char S[100];
 int  k;
